-- accy_accessory definition

CREATE TABLE `accy_accessory`
(
    `id`                     bigint(20) NOT NULL AUTO_INCREMENT,
    `equipment_accessory_id` bigint(20) DEFAULT NULL COMMENT '设备辅材id',
    `equipment_code`         varchar(50)  DEFAULT NULL,
    `material_id`            bigint(20) NOT NULL COMMENT '物料id',
    `material_type`          varchar(50) NOT NULL COMMENT '物料类型',
    `material_name`          varchar(50) NOT NULL COMMENT '物料名称',
    `material_code`          varchar(50) NOT NULL COMMENT '物料料号',
    `batch_code`             varchar(255) DEFAULT NULL COMMENT '批次号',
    `unique_code`            varchar(50)  DEFAULT NULL COMMENT '唯一码',
    `number`                 int(11) NOT NULL COMMENT '数量',
    `accessory_status`       int(11) DEFAULT NULL COMMENT '物料状态',
    `status`                 tinyint(4) NOT NULL COMMENT '物料状态（0：待上机，1：上机中，2：退库待确认，3：已退库）',
    `warn_status`            tinyint(4) NOT NULL COMMENT '预警状态（0：未预警，1：已预警）',
    `supplier`               varchar(50)  DEFAULT NULL COMMENT '供应商',
    `xh`                     varchar(50)  DEFAULT NULL COMMENT '型号',
    `th`                     varchar(50)  DEFAULT NULL COMMENT '图号',
    `unit`                   varchar(50)  DEFAULT NULL COMMENT '单位',
    `life_unit`              varchar(50)  DEFAULT NULL COMMENT '寿命单位',
    `rated_life`             int(11) NOT NULL COMMENT '额定寿命',
    `useful_life`            int(11) NOT NULL COMMENT '已使用寿命',
    `remaining_life`         int(11) NOT NULL COMMENT '剩余寿命',
    `warn_life`              int(11) DEFAULT NULL COMMENT '预警寿命',
    `upper_time`             datetime     DEFAULT NULL COMMENT '上机时间',
    `down_time`              datetime     DEFAULT NULL COMMENT '下机时间',
    `warn_time`              datetime     DEFAULT NULL COMMENT '预警时间',
    `user_create`            varchar(20) NOT NULL COMMENT '创建/修改人',
    `gmt_create`             datetime    NOT NULL COMMENT '创建/修改时间',
    `specification`          varchar(50)  DEFAULT NULL COMMENT '规格',
    `life_type`              int(1) DEFAULT '0' COMMENT '寿命类型（0：普通，1：炉次）',
    `user_update`            varchar(50)  DEFAULT NULL,
    `gmt_update`             datetime     DEFAULT NULL,
    PRIMARY KEY (`id`),
    UNIQUE KEY `idx_unique_code` (`unique_code`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 AVG_ROW_LENGTH=682;


-- accy_apply_record definition

CREATE TABLE `accy_apply_record`
(
    `id`             bigint(20) NOT NULL AUTO_INCREMENT,
    `code`           varchar(50)  DEFAULT NULL COMMENT '申请单号',
    `equipment_code` varchar(50) NOT NULL COMMENT '设备号',
    `material_id`    bigint(20) DEFAULT NULL COMMENT '物料id',
    `material_type`  varchar(50) NOT NULL COMMENT '物料类型',
    `material_name`  varchar(50) NOT NULL COMMENT '物料名称',
    `material_code`  varchar(50) NOT NULL COMMENT '物料料号',
    `number`         int(11) NOT NULL COMMENT '数量',
    `unit`           varchar(50) NOT NULL COMMENT '单位',
    `reason`         varchar(255) DEFAULT NULL COMMENT '原因',
    `status`         tinyint(4) DEFAULT NULL COMMENT '状态（0:待出库，1:已出库，2:已拒绝）',
    `user_create`    varchar(20) NOT NULL COMMENT '申请人',
    `gmt_create`     datetime    NOT NULL COMMENT '申请时间',
    `user_confirm`   varchar(20)  DEFAULT NULL COMMENT '确认人',
    `gmt_confirm`    datetime     DEFAULT NULL COMMENT '确认时间',
    PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 AVG_ROW_LENGTH=192 COMMENT='申请记录';


-- accy_equipment_accessory definition

CREATE TABLE `accy_equipment_accessory`
(
    `id`                     bigint(20) NOT NULL AUTO_INCREMENT,
    `area`                   varchar(50) DEFAULT NULL COMMENT '模块',
    `series`                 varchar(50) DEFAULT NULL COMMENT '系列',
    `equipment_code`         varchar(50) DEFAULT NULL COMMENT '设备号',
    `early_warning_count`    int(11) DEFAULT NULL COMMENT '预警数量',
    `waiting_boarding_count` int(11) DEFAULT NULL COMMENT '待上机数量',
    `boarding_count`         int(11) DEFAULT NULL COMMENT '上机中数量',
    `stock_return_count`     int(11) DEFAULT NULL COMMENT '退库待确认数量',
    `waiting_outbound_count` int(11) DEFAULT NULL COMMENT '待出库数量',
    `user_create`            varchar(20) NOT NULL COMMENT '创建/修改人',
    `gmt_create`             datetime    NOT NULL COMMENT '创建/修改时间',
    PRIMARY KEY (`id`),
    UNIQUE KEY `idx_equipment_code` (`equipment_code`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 AVG_ROW_LENGTH=348;


-- accy_life_resume definition

CREATE TABLE `accy_life_resume`
(
    `id`             bigint(20) NOT NULL AUTO_INCREMENT,
    `accessory_id`   bigint(20) DEFAULT NULL COMMENT '辅材id',
    `operate_time`   datetime     DEFAULT NULL COMMENT '操作时间',
    `type`           tinyint(4) DEFAULT NULL COMMENT '类型（0: 上机，1：下机）',
    `message`        text COMMENT '信息（html）',
    `user_create`    varchar(255) DEFAULT NULL COMMENT '操作人',
    `equipment_code` varchar(50)  DEFAULT NULL COMMENT '设备编号',
    PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 AVG_ROW_LENGTH=887;


-- accy_replacement_record definition

CREATE TABLE `accy_replacement_record`
(
    `id`                 int(11) NOT NULL AUTO_INCREMENT,
    `equipment_code`     varchar(50)  DEFAULT NULL COMMENT '设备号',
    `material_type`      varchar(50)  NOT NULL COMMENT '物料类型',
    `material_name`      varchar(50)  NOT NULL COMMENT '物料名称',
    `material_code`      varchar(50)  NOT NULL COMMENT '物料料号',
    `batch_code`         varchar(255) DEFAULT NULL COMMENT '批次号',
    `unique_code`        varchar(50)  DEFAULT NULL COMMENT '唯一码',
    `number`             int(11) NOT NULL COMMENT '数量',
    `specification`      varchar(130) NOT NULL COMMENT '规格',
    `supplier`           varchar(50)  DEFAULT NULL COMMENT '供应商',
    `rated_life`         int(11) NOT NULL COMMENT '额定寿命',
    `useful_life`        int(11) NOT NULL COMMENT '已使用寿命',
    `remaining_life`     int(11) NOT NULL COMMENT '剩余寿命',
    `warn_life`          int(11) DEFAULT NULL COMMENT '预警寿命',
    `material_type_old`  varchar(50)  NOT NULL COMMENT '物料类型（前）',
    `material_name_old`  varchar(50)  NOT NULL COMMENT '物料名称（前）',
    `material_code_old`  varchar(50)  NOT NULL COMMENT '物料料号（前）',
    `batch_code_old`     varchar(255) DEFAULT NULL COMMENT '批次号（前）',
    `unique_code_old`    varchar(50)  DEFAULT NULL COMMENT '唯一码（前）',
    `number_old`         int(11) NOT NULL COMMENT '数量（前）',
    `specification_old`  varchar(130) NOT NULL COMMENT '规格（前）',
    `supplier_old`       varchar(50)  DEFAULT NULL COMMENT '供应商（前）',
    `rated_life_old`     int(11) NOT NULL COMMENT '额定寿命（前）',
    `useful_life_old`    int(11) NOT NULL COMMENT '已使用寿命（前）',
    `remaining_life_old` int(11) NOT NULL COMMENT '剩余寿命（前）',
    `warn_life_old`      int(11) DEFAULT NULL COMMENT '预警寿命（前）',
    `replace_time`       datetime     DEFAULT NULL COMMENT '换下时间',
    `replace_user`       varchar(50)  DEFAULT NULL COMMENT '换下人',
    `change_time`        datetime     DEFAULT NULL COMMENT '换上时间',
    `change_user`        varchar(50)  DEFAULT NULL COMMENT '换上人',
    `remark`             varchar(255) DEFAULT NULL COMMENT '备注',
    `reason`             varchar(50)  DEFAULT NULL COMMENT '更换原因',
    PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4;


-- accy_return_record definition

CREATE TABLE `accy_return_record`
(
    `id`             bigint(20) NOT NULL AUTO_INCREMENT,
    `accessory_id`   bigint(20) DEFAULT NULL COMMENT '辅材id',
    `code`           varchar(50)  NOT NULL COMMENT '退料单号',
    `equipment_code` varchar(50)  NOT NULL COMMENT '设备号',
    `material_type`  varchar(50)  NOT NULL COMMENT '物料类型',
    `material_name`  varchar(50)  NOT NULL COMMENT '物料名称',
    `material_code`  varchar(50)  NOT NULL COMMENT '物料料号',
    `batch_code`     varchar(255) DEFAULT NULL COMMENT '批次号',
    `unique_code`    varchar(50)  DEFAULT NULL COMMENT '唯一码',
    `number`         int(11) NOT NULL COMMENT '数量',
    `use_status`     varchar(255) DEFAULT NULL COMMENT '使用状态（0：待上机，1：上机中）',
    `status`         tinyint(4) DEFAULT NULL COMMENT '退库状态（0：待接收，1：已接收，2：已取消，3：已拒收）',
    `specification`  varchar(130) NOT NULL COMMENT '规格',
    `unit`           varchar(50)  NOT NULL COMMENT '单位',
    `supplier`       varchar(50)  DEFAULT NULL COMMENT '供应商',
    `rated_life`     int(11) NOT NULL COMMENT '额定寿命',
    `useful_life`    int(11) NOT NULL COMMENT '已使用寿命',
    `remaining_life` int(11) DEFAULT NULL COMMENT '剩余寿命',
    `reason`         varchar(255) DEFAULT NULL COMMENT '退库原因',
    `user_create`    varchar(20)  NOT NULL COMMENT '申请人',
    `gmt_create`     datetime     NOT NULL COMMENT '申请时间',
    `user_confirm`   varchar(20)  DEFAULT NULL COMMENT '确认人',
    `gmt_confirm`    datetime     DEFAULT NULL COMMENT '确认时间',
    PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8mb4 AVG_ROW_LENGTH=963;