package com.sw.mes.itemclean.clean.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynmaicQueryWrapper;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.itemclean.clean.dto.MaterialCleaningBasketDto;
import com.sw.mes.itemclean.clean.domain.MaterialCleaningBasket;
import com.sw.mes.itemclean.clean.mapper.MaterialCleaningBasketMapper;
import com.sw.mes.itemclean.clean.service.MaterialCleaningBasketService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


@RestController
@RequestMapping("/clean/materialCleaningBasket")
@Slf4j
@Api(tags = "清洗花篮")
public class MaterialCleaningBasketController {
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private MaterialCleaningBasketService materialCleaningBasketService;

    @Autowired
    private MaterialCleaningBasketMapper materialCleaningBasketMapper;

    @ApiOperation(value = "分页查询列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<MaterialCleaningBasketDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        //分页查询
        QueryWrapper<MaterialCleaningBasket> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, MaterialCleaningBasket.class);
        Page<MaterialCleaningBasket> listPage = materialCleaningBasketService.page(new Page<>(requestPage.getPage(), requestPage.getRows()), queryWrapper);
        List<MaterialCleaningBasketDto> dtos = listPage.getRecords().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, listPage.getTotal());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "获取所有列表")
    @GetMapping("/listAll")
    public Result<List<MaterialCleaningBasketDto>> listAll(ServletRequest request) {
        QueryWrapper<MaterialCleaningBasket> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, MaterialCleaningBasket.class);
        List<MaterialCleaningBasket> materialCleaningBaskets = materialCleaningBasketService.list(queryWrapper);
        List<MaterialCleaningBasketDto> materialCleaningBasketDtos = materialCleaningBaskets.stream()
                .map(materialCleaningBasket -> toLazyDto(materialCleaningBasket))
                .collect(Collectors.toList());
        return ResultGenerator.success(materialCleaningBasketDtos);
    }

    @ApiOperation(value = "详细信息")
    @GetMapping("/{id}")
    public Result<MaterialCleaningBasketDto> get(@PathVariable @ApiParam(value="设备类型id") Integer id) {
        MaterialCleaningBasket materialCleaningBasket = materialCleaningBasketService.getById(id);
        if (null == materialCleaningBasket)
            return ResultGenerator.fail("该对象不存在");
        return ResultGenerator.success(toEagerDto(materialCleaningBasket));
    }

    @ApiOperation(value = "创建")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "Dto对象") MaterialCleaningBasketDto materialCleaningBasketDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        MaterialCleaningBasket materialCleaningBasket = fromDto(materialCleaningBasketDto);
        materialCleaningBasketService.saveOrUpdate(materialCleaningBasket);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "Dto对象") MaterialCleaningBasketDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
        MaterialCleaningBasket entity = materialCleaningBasketService.getById(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            MaterialCleaningBasket entityfromFto = fromDto(dto);
            BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
            materialCleaningBasketService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "删除")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
        for (Integer id : ids) {
            materialCleaningBasketMapper.deleteById(id);
        }
        return ResultGenerator.success();
    }

    public MaterialCleaningBasketDto toLazyDto(MaterialCleaningBasket entity) {
        return modelMapper.map(entity, MaterialCleaningBasketDto.class);
    }

    public MaterialCleaningBasketDto toEagerDto(MaterialCleaningBasket entity) {
        MaterialCleaningBasketDto dto = modelMapper.map(entity, MaterialCleaningBasketDto.class);
        return dto;
    }

    public MaterialCleaningBasket fromDto(MaterialCleaningBasketDto dto) {
        return modelMapper.map(dto, MaterialCleaningBasket.class);
    }
}
