package com.sw.mes.itemclean.clean.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynmaicQueryWrapper;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.itemclean.clean.dto.MaterialCleaningDetailDto;
import com.sw.mes.itemclean.clean.domain.MaterialCleaningDetail;
import com.sw.mes.itemclean.clean.mapper.MaterialCleaningDetailMapper;
import com.sw.mes.itemclean.clean.service.MaterialCleaningDetailService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/clean/materialCleaningDetail")
@Slf4j
@Api(tags = "清洗明细(对应料箱明细)")
public class MaterialCleaningDetailController {
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private MaterialCleaningDetailService materialCleaningDetailService;

    @Autowired
    private MaterialCleaningDetailMapper materialCleaningDetailMapper;

    @ApiOperation(value = "分页查询列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<MaterialCleaningDetailDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        //分页查询
        QueryWrapper<MaterialCleaningDetail> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, MaterialCleaningDetail.class);
        Page<MaterialCleaningDetail> listPage = materialCleaningDetailService.page(new Page<>(requestPage.getPage(), requestPage.getRows()), queryWrapper);
        List<MaterialCleaningDetailDto> dtos = listPage.getRecords().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, listPage.getTotal());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "获取所有列表")
    @GetMapping("/listAll")
    public Result<List<MaterialCleaningDetailDto>> listAll(ServletRequest request) {
        QueryWrapper<MaterialCleaningDetail> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, MaterialCleaningDetail.class);
        List<MaterialCleaningDetail> materialCleaningDetails = materialCleaningDetailService.list(queryWrapper);
        List<MaterialCleaningDetailDto> materialCleaningDetailDtos = materialCleaningDetails.stream()
                .map(materialCleaningDetail -> toLazyDto(materialCleaningDetail))
                .collect(Collectors.toList());
        return ResultGenerator.success(materialCleaningDetailDtos);
    }

    @ApiOperation(value = "详细信息")
    @GetMapping("/{id}")
    public Result<MaterialCleaningDetailDto> get(@PathVariable @ApiParam(value="设备类型id") Integer id) {
        MaterialCleaningDetail materialCleaningDetail = materialCleaningDetailService.getById(id);
        if (null == materialCleaningDetail)
            return ResultGenerator.fail("该对象不存在");
        return ResultGenerator.success(toEagerDto(materialCleaningDetail));
    }

    @ApiOperation(value = "创建")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "Dto对象") MaterialCleaningDetailDto materialCleaningDetailDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        MaterialCleaningDetail materialCleaningDetail = fromDto(materialCleaningDetailDto);
        materialCleaningDetailService.saveOrUpdate(materialCleaningDetail);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "Dto对象") MaterialCleaningDetailDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
        MaterialCleaningDetail entity = materialCleaningDetailService.getById(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            MaterialCleaningDetail entityfromFto = fromDto(dto);
            BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
            materialCleaningDetailService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "删除")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
        for (Integer id : ids) {
            materialCleaningDetailMapper.deleteById(id);
        }
        return ResultGenerator.success();
    }

    public MaterialCleaningDetailDto toLazyDto(MaterialCleaningDetail entity) {
        return modelMapper.map(entity, MaterialCleaningDetailDto.class);
    }

    public MaterialCleaningDetailDto toEagerDto(MaterialCleaningDetail entity) {
        MaterialCleaningDetailDto dto = modelMapper.map(entity, MaterialCleaningDetailDto.class);
        return dto;
    }

    public MaterialCleaningDetail fromDto(MaterialCleaningDetailDto dto) {
        return modelMapper.map(dto, MaterialCleaningDetail.class);
    }
}
