package com.sw.mes.itemclean.clean.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynmaicQueryWrapper;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.itemclean.clean.dto.MaterialCleaningDownDto;
import com.sw.mes.itemclean.clean.domain.MaterialCleaningDown;
import com.sw.mes.itemclean.clean.mapper.MaterialCleaningDownMapper;
import com.sw.mes.itemclean.clean.service.MaterialCleaningDownService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


@RestController
@RequestMapping("/clean/materialCleaningDown")
@Slf4j
@Api(tags = "清洗下料")
public class MaterialCleaningDownController {
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private MaterialCleaningDownService materialCleaningDownService;

    @Autowired
    private MaterialCleaningDownMapper materialCleaningDownMapper;

    @ApiOperation(value = "分页查询列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<MaterialCleaningDownDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        //分页查询
        QueryWrapper<MaterialCleaningDown> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, MaterialCleaningDown.class);
        Page<MaterialCleaningDown> listPage = materialCleaningDownService.page(new Page<>(requestPage.getPage(), requestPage.getRows()), queryWrapper);
        List<MaterialCleaningDownDto> dtos = listPage.getRecords().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, listPage.getTotal());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "获取所有列表")
    @GetMapping("/listAll")
    public Result<List<MaterialCleaningDownDto>> listAll(ServletRequest request) {
        QueryWrapper<MaterialCleaningDown> queryWrapper =  DynmaicQueryWrapper.searchFilter(request, MaterialCleaningDown.class);
        List<MaterialCleaningDown> materialCleaningDowns = materialCleaningDownService.list(queryWrapper);
        List<MaterialCleaningDownDto> materialCleaningDownDtos = materialCleaningDowns.stream()
                .map(materialCleaningDown -> toLazyDto(materialCleaningDown))
                .collect(Collectors.toList());
        return ResultGenerator.success(materialCleaningDownDtos);
    }

    @ApiOperation(value = "详细信息")
    @GetMapping("/{id}")
    public Result<MaterialCleaningDownDto> get(@PathVariable @ApiParam(value="设备类型id") Integer id) {
        MaterialCleaningDown materialCleaningDown = materialCleaningDownService.getById(id);
        if (null == materialCleaningDown)
            return ResultGenerator.fail("该对象不存在");
        return ResultGenerator.success(toEagerDto(materialCleaningDown));
    }

    @ApiOperation(value = "创建")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "Dto对象") MaterialCleaningDownDto materialCleaningDownDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        MaterialCleaningDown materialCleaningDown = fromDto(materialCleaningDownDto);
        materialCleaningDownService.saveOrUpdate(materialCleaningDown);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "Dto对象") MaterialCleaningDownDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
        MaterialCleaningDown entity = materialCleaningDownService.getById(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            MaterialCleaningDown entityfromFto = fromDto(dto);
            BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
            materialCleaningDownService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "删除")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
        for (Integer id : ids) {
            materialCleaningDownMapper.deleteById(id);
        }
        return ResultGenerator.success();
    }

    public MaterialCleaningDownDto toLazyDto(MaterialCleaningDown entity) {
        return modelMapper.map(entity, MaterialCleaningDownDto.class);
    }

    public MaterialCleaningDownDto toEagerDto(MaterialCleaningDown entity) {
        MaterialCleaningDownDto dto = modelMapper.map(entity, MaterialCleaningDownDto.class);
        return dto;
    }

    public MaterialCleaningDown fromDto(MaterialCleaningDownDto dto) {
        return modelMapper.map(dto, MaterialCleaningDown.class);
    }
}
