package com.sw.mes.fb.aio.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.fb.aio.dto.EapCutCraftShortDto;
import com.sw.mes.fb.aio.entity.EapCutCraftShort;
import com.sw.mes.fb.aio.service.EapCutCraftShortService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/eapCutCraftShort")
@Slf4j
@Api(description = "截断工艺数据上传记录(短棒)信息接口")
public class EapCutCraftShortController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private EapCutCraftShortService eapCutCraftShortService;

	@ApiOperation(value = "分页查询截断工艺数据上传记录(短棒)信息列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<EapCutCraftShortDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<EapCutCraftShort> specification = DynamicSpecifications.bySearchFilter(request, EapCutCraftShort.class);
    	Page<EapCutCraftShort> page = eapCutCraftShortService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<EapCutCraftShortDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有截断工艺数据上传记录(短棒)信息")
	@GetMapping("/listAll")
	public Result<List<EapCutCraftShortDto>> listAll(ServletRequest request) {
		Specification<EapCutCraftShort> specification = DynamicSpecifications.bySearchFilter(request, EapCutCraftShort.class);
		List<EapCutCraftShort> eapCutCraftShorts = eapCutCraftShortService.findByExampleSort(specification, null);
		List<EapCutCraftShortDto> eapCutCraftShortDtos = eapCutCraftShorts.stream()
					.map(eapCutCraftShort -> toLazyDto(eapCutCraftShort))
					.collect(Collectors.toList());
		return ResultGenerator.success(eapCutCraftShortDtos);
	}

	@ApiOperation(value = "截断工艺数据上传记录(短棒)信息详细信息")
	@GetMapping("/{id}")
	public Result<EapCutCraftShortDto> get(@PathVariable @ApiParam(value="截断工艺数据上传记录(短棒)信息id") Long id) {
		EapCutCraftShort eapCutCraftShort = eapCutCraftShortService.get(id);
		if (null == eapCutCraftShort)
			return ResultGenerator.fail("该截断工艺数据上传记录(短棒)信息不存在");
		return ResultGenerator.success(toEagerDto(eapCutCraftShort));
	}

	@ApiOperation(value = "创建截断工艺数据上传记录(短棒)信息")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "截断工艺数据上传记录(短棒)信息对象") EapCutCraftShortDto eapCutCraftShortDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		EapCutCraftShort eapCutCraftShort = fromDto(eapCutCraftShortDto);
		eapCutCraftShortService.saveOrUpdate(eapCutCraftShort);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新截断工艺数据上传记录(短棒)信息")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "截断工艺数据上传记录(短棒)信息") EapCutCraftShortDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            EapCutCraftShort entity = eapCutCraftShortService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            EapCutCraftShort entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			eapCutCraftShortService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除截断工艺数据上传记录(短棒)信息")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				eapCutCraftShortService.delete(id);
		}
		return ResultGenerator.success();
	}

	public EapCutCraftShortDto toLazyDto(EapCutCraftShort entity) {
		return modelMapper.map(entity, EapCutCraftShortDto.class);
	}

	public EapCutCraftShortDto toEagerDto(EapCutCraftShort entity) {
		EapCutCraftShortDto dto = modelMapper.map(entity, EapCutCraftShortDto.class);
		return dto;
	}

	public EapCutCraftShort fromDto(EapCutCraftShortDto dto) {
		return modelMapper.map(dto, EapCutCraftShort.class);
	}


}
