package com.sw.mes.fb.checkout.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.fb.checkout.dto.CooperateDataDto;
import com.sw.mes.fb.checkout.entity.CooperateData;
import com.sw.mes.fb.checkout.service.CooperateDataService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@RestController
@RequestMapping("/CooperateData")
@Slf4j
@Api(description = "方棒配棒记录接口")
public class CooperateDataController {
    @Autowired
    ModelMapper modelMapper;

    @Autowired
    private CooperateDataService cooperateDataService;

    @ApiOperation(value = "分页查询方棒配棒记录列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
            @ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
    })
    @GetMapping("/listByPage")
    public Result<ResponsePage<CooperateDataDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
        Specification<CooperateData> specification = DynamicSpecifications.bySearchFilter(request, CooperateData.class);
        Page<CooperateData> page = cooperateDataService.findByExample(specification, PageUtil.createPageable(requestPage));
        List<CooperateDataDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
        ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
    }

    @ApiOperation(value = "获取所有方棒配棒记录")
    @GetMapping("/listAll")
    public Result<List<CooperateDataDto>> listAll(ServletRequest request) {
        Specification<CooperateData> specification = DynamicSpecifications.bySearchFilter(request, CooperateData.class);
        List<CooperateData> cooperateDatas = cooperateDataService.findByExampleSort(specification, null);
        List<CooperateDataDto> cooperateDataDtos = cooperateDatas.stream()
                .map(cooperateData -> toLazyDto(cooperateData))
                .collect(Collectors.toList());
        return ResultGenerator.success(cooperateDataDtos);
    }

    @ApiOperation(value = "方棒配棒记录详细信息")
    @GetMapping("/{id}")
    public Result<CooperateDataDto> get(@PathVariable @ApiParam(value = "方棒配棒记录id") Long id) {
        CooperateData cooperateData = cooperateDataService.get(id);
        if (null == cooperateData) {
            return ResultGenerator.fail("该方棒配棒记录不存在");
        }
        return ResultGenerator.success(toEagerDto(cooperateData));
    }

    @ApiOperation(value = "通过方棒编号查询配棒信息")
    @GetMapping("/findBySectionNoFb")
    public Result findBySectionNoFb(@RequestParam String fbSectionNo) {
        return ResultGenerator.success(cooperateDataService.findBySectionNoFb(fbSectionNo));
    }

    @ApiOperation(value = "创建方棒配棒记录")
    @PostMapping("/create")
    public Result create(@RequestBody @Valid @ApiParam(value = "方棒配棒记录对象") CooperateDataDto cooperateDataDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        CooperateData cooperateData = fromDto(cooperateDataDto);
        cooperateDataService.saveOrUpdate(cooperateData);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新方棒配棒记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "方棒配棒记录") CooperateDataDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if (dto.getId() == null) {
            return ResultGenerator.fail("id不能为空");
        }
        CooperateData entity = cooperateDataService.get(dto.getId());
        if (entity == null) {
            return ResultGenerator.fail("原信息不存在");
        }
        try {
            CooperateData entityfromFto = fromDto(dto);
            BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
            cooperateDataService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

    @ApiOperation(value = "删除方棒配棒记录")
    @PostMapping("/delete")
    public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
        for (Integer id : ids) {
            cooperateDataService.delete(id);
        }
        return ResultGenerator.success();
    }

    public CooperateDataDto toLazyDto(CooperateData entity) {
        return modelMapper.map(entity, CooperateDataDto.class);
    }

    public CooperateDataDto toEagerDto(CooperateData entity) {
        CooperateDataDto dto = modelMapper.map(entity, CooperateDataDto.class);
        return dto;
    }

    public CooperateData fromDto(CooperateDataDto dto) {
        return modelMapper.map(dto, CooperateData.class);
    }


}
