package com.sw.mes.fb.maketray.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.fb.maketray.dto.MakeTrayItemDto;
import com.sw.mes.fb.maketray.entity.MakeTrayItem;
import com.sw.mes.fb.maketray.service.MakeTrayItemService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/makeTrayItem")
@Slf4j
@Api(description = "打拖明细接口")
public class MakeTrayItemController {
	@Autowired
    ModelMapper modelMapper;

	@Autowired
	private MakeTrayItemService makeTrayItemService;

	@ApiOperation(value = "分页查询打拖明细列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<MakeTrayItemDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<MakeTrayItem> specification = DynamicSpecifications.bySearchFilter(request, MakeTrayItem.class);
    	Page<MakeTrayItem> page = makeTrayItemService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<MakeTrayItemDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有打拖明细")
	@GetMapping("/listAll")
	public Result<List<MakeTrayItemDto>> listAll(ServletRequest request) {
		Specification<MakeTrayItem> specification = DynamicSpecifications.bySearchFilter(request, MakeTrayItem.class);
		List<MakeTrayItem> makeTrayItems = makeTrayItemService.findByExampleSort(specification, null);
		List<MakeTrayItemDto> makeTrayItemDtos = makeTrayItems.stream()
					.map(makeTrayItem -> toLazyDto(makeTrayItem))
					.collect(Collectors.toList());
		return ResultGenerator.success(makeTrayItemDtos);
	}

	@ApiOperation(value = "打拖明细详细信息")
	@GetMapping("/{id}")
	public Result<MakeTrayItemDto> get(@PathVariable @ApiParam(value="打拖明细id") Integer id) {
		MakeTrayItem makeTrayItem = makeTrayItemService.get(id);
		if (null == makeTrayItem)
			return ResultGenerator.fail("该打拖明细不存在");
		return ResultGenerator.success(toEagerDto(makeTrayItem));
	}

	@ApiOperation(value = "创建打拖明细")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "打拖明细对象") MakeTrayItemDto makeTrayItemDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		MakeTrayItem makeTrayItem = fromDto(makeTrayItemDto);
		makeTrayItemService.saveOrUpdate(makeTrayItem);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新打拖明细")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "打拖明细") MakeTrayItemDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            MakeTrayItem entity = makeTrayItemService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            MakeTrayItem entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			makeTrayItemService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除打拖明细")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				makeTrayItemService.delete(id);
		}
		return ResultGenerator.success();
	}

	public MakeTrayItemDto toLazyDto(MakeTrayItem entity) {
		return modelMapper.map(entity, MakeTrayItemDto.class);
	}

	public MakeTrayItemDto toEagerDto(MakeTrayItem entity) {
		MakeTrayItemDto dto = modelMapper.map(entity, MakeTrayItemDto.class);
		return dto;
	}

	public MakeTrayItem fromDto(MakeTrayItemDto dto) {
		return modelMapper.map(dto, MakeTrayItem.class);
	}


}
