package com.sw.mes.grade.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.grade.entity.RuleTest;
import com.sw.mes.grade.service.RuleTestService;
import com.sw.mes.grade.dto.RuleTestDto;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/ruleTest")
@Slf4j
@Api(description = "评级测试记录接口")
public class RuleTestController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private RuleTestService ruleTestService;

	@ApiOperation(value = "分页查询评级测试记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<RuleTestDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<RuleTest> specification = DynamicSpecifications.bySearchFilter(request, RuleTest.class);
    	Page<RuleTest> page = ruleTestService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<RuleTestDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有评级测试记录")
	@GetMapping("/listAll")
	public Result<List<RuleTestDto>> listAll(ServletRequest request) {
		Specification<RuleTest> specification = DynamicSpecifications.bySearchFilter(request, RuleTest.class);
		List<RuleTest> ruleTests = ruleTestService.findByExampleSort(specification, null);
		List<RuleTestDto> ruleTestDtos = ruleTests.stream()
					.map(ruleTest -> toLazyDto(ruleTest))
					.collect(Collectors.toList());
		return ResultGenerator.success(ruleTestDtos);
	}

	@ApiOperation(value = "评级测试记录详细信息")
	@GetMapping("/{id}")
	public Result<RuleTestDto> get(@PathVariable @ApiParam(value="评级测试记录id") Integer id) {
		RuleTest ruleTest = ruleTestService.get(id);
		if (null == ruleTest)
			return ResultGenerator.fail("该评级测试记录不存在");
		return ResultGenerator.success(toEagerDto(ruleTest));
	}

	@ApiOperation(value = "创建评级测试记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "评级测试记录对象") RuleTestDto ruleTestDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		RuleTest ruleTest = fromDto(ruleTestDto);
		ruleTestService.saveOrUpdate(ruleTest);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新评级测试记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "评级测试记录") RuleTestDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            RuleTest entity = ruleTestService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            RuleTest entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			ruleTestService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除评级测试记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
		for (Long id : ids) {
				ruleTestService.delete(id);
		}
		return ResultGenerator.success();
	}

	public RuleTestDto toLazyDto(RuleTest entity) {
		return modelMapper.map(entity, RuleTestDto.class);
	}

	public RuleTestDto toEagerDto(RuleTest entity) {
		RuleTestDto dto = modelMapper.map(entity, RuleTestDto.class);
		return dto;
	}

	public RuleTest fromDto(RuleTestDto dto) {
		return modelMapper.map(dto, RuleTest.class);
	}


}
