package com.sw.mes.plan.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.plan.entity.FbOrderMatchingInfo;
import com.sw.mes.plan.dto.FbOrderMatchingInfoDto;
import com.sw.mes.plan.service.FbOrderMatchingInfoService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/fbOrderMatchingInfo")
@Slf4j
@Api(description = "FbOrderMatchingInfo接口")
public class FbOrderMatchingInfoController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private FbOrderMatchingInfoService fbOrderMatchingInfoService;

	@ApiOperation(value = "分页查询订单匹配记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<FbOrderMatchingInfoDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<FbOrderMatchingInfo> specification = DynamicSpecifications.bySearchFilter(request, FbOrderMatchingInfo.class);
    	Page<FbOrderMatchingInfo> page = fbOrderMatchingInfoService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<FbOrderMatchingInfoDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有订单匹配记录")
	@GetMapping("/listAll")
	public Result<List<FbOrderMatchingInfoDto>> listAll(ServletRequest request) {
		Specification<FbOrderMatchingInfo> specification = DynamicSpecifications.bySearchFilter(request, FbOrderMatchingInfo.class);
		List<FbOrderMatchingInfo> fbOrderMatchingInfos = fbOrderMatchingInfoService.findByExampleSort(specification, null);
		List<FbOrderMatchingInfoDto> fbOrderMatchingInfoDtos = fbOrderMatchingInfos.stream()
					.map(fbOrderMatchingInfo -> toLazyDto(fbOrderMatchingInfo))
					.collect(Collectors.toList());
		return ResultGenerator.success(fbOrderMatchingInfoDtos);
	}

	@ApiOperation(value = "FbOrderMatchingInfo详细信息")
	@GetMapping("/{id}")
	public Result<FbOrderMatchingInfoDto> get(@PathVariable @ApiParam(value="FbOrderMatchingInfoid") Integer id) {
		FbOrderMatchingInfo fbOrderMatchingInfo = fbOrderMatchingInfoService.get(id);
		if (null == fbOrderMatchingInfo)
			return ResultGenerator.fail("该FbOrderMatchingInfo不存在");
		return ResultGenerator.success(toEagerDto(fbOrderMatchingInfo));
	}

	@ApiOperation(value = "创建订单匹配记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "FbOrderMatchingInfo对象") FbOrderMatchingInfoDto fbOrderMatchingInfoDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		FbOrderMatchingInfo fbOrderMatchingInfo = fromDto(fbOrderMatchingInfoDto);
		fbOrderMatchingInfoService.saveOrUpdate(fbOrderMatchingInfo);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新订单匹配记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "FbOrderMatchingInfo") FbOrderMatchingInfoDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            FbOrderMatchingInfo entity = fbOrderMatchingInfoService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            FbOrderMatchingInfo entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			fbOrderMatchingInfoService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除订单匹配记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				fbOrderMatchingInfoService.delete(id);
		}
		return ResultGenerator.success();
	}

	public FbOrderMatchingInfoDto toLazyDto(FbOrderMatchingInfo entity) {
		return modelMapper.map(entity, FbOrderMatchingInfoDto.class);
	}

	public FbOrderMatchingInfoDto toEagerDto(FbOrderMatchingInfo entity) {
		FbOrderMatchingInfoDto dto = modelMapper.map(entity, FbOrderMatchingInfoDto.class);
		return dto;
	}

	public FbOrderMatchingInfo fromDto(FbOrderMatchingInfoDto dto) {
		return modelMapper.map(dto, FbOrderMatchingInfo.class);
	}


}
