package com.sw.mes.production.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.production.dto.FormulaProductSchemeDetailDto;
import com.sw.mes.production.entity.FormulaProductSchemeDetail;
import com.sw.mes.production.service.FormulaProductSchemeDetailService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/productSchemeDetail")
@Slf4j
@Api(description = "产品配方明细接口")
public class FormulaProductSchemeDetailController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private FormulaProductSchemeDetailService formulaProductSchemeDetailService;

	@ApiOperation(value = "分页查询产品配方明细列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<FormulaProductSchemeDetailDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<FormulaProductSchemeDetail> specification = DynamicSpecifications.bySearchFilter(request, FormulaProductSchemeDetail.class);
    	Page<FormulaProductSchemeDetail> page = formulaProductSchemeDetailService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<FormulaProductSchemeDetailDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有产品配方明细")
	@GetMapping("/listAll")
	public Result<List<FormulaProductSchemeDetailDto>> listAll(ServletRequest request) {
		Specification<FormulaProductSchemeDetail> specification = DynamicSpecifications.bySearchFilter(request, FormulaProductSchemeDetail.class);
		List<FormulaProductSchemeDetail> formulaProductSchemeDetails = formulaProductSchemeDetailService.findByExampleSort(specification, null);
		List<FormulaProductSchemeDetailDto> formulaProductSchemeDetailDtos = formulaProductSchemeDetails.stream()
					.map(formulaProductSchemeDetail -> toLazyDto(formulaProductSchemeDetail))
					.collect(Collectors.toList());
		return ResultGenerator.success(formulaProductSchemeDetailDtos);
	}

	@ApiOperation(value = "产品配方明细详细信息")
	@GetMapping("/{id}")
	public Result<FormulaProductSchemeDetailDto> get(@PathVariable @ApiParam(value="产品配方明细id") Long id) {
		FormulaProductSchemeDetail formulaProductSchemeDetail = formulaProductSchemeDetailService.get(id);
		if (null == formulaProductSchemeDetail)
			return ResultGenerator.fail("该产品配方明细不存在");
		return ResultGenerator.success(toEagerDto(formulaProductSchemeDetail));
	}

	@ApiOperation(value = "创建产品配方明细")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "产品配方明细对象") FormulaProductSchemeDetailDto formulaProductSchemeDetailDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		FormulaProductSchemeDetail formulaProductSchemeDetail = fromDto(formulaProductSchemeDetailDto);
		formulaProductSchemeDetailService.saveOrUpdate(formulaProductSchemeDetail);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新产品配方明细")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "产品配方明细") FormulaProductSchemeDetailDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            FormulaProductSchemeDetail entity = formulaProductSchemeDetailService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            FormulaProductSchemeDetail entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			formulaProductSchemeDetailService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除产品配方明细")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				formulaProductSchemeDetailService.delete(id);
		}
		return ResultGenerator.success();
	}

	public FormulaProductSchemeDetailDto toLazyDto(FormulaProductSchemeDetail entity) {
		return modelMapper.map(entity, FormulaProductSchemeDetailDto.class);
	}

	public FormulaProductSchemeDetailDto toEagerDto(FormulaProductSchemeDetail entity) {
		FormulaProductSchemeDetailDto dto = modelMapper.map(entity, FormulaProductSchemeDetailDto.class);
		return dto;
	}

	public FormulaProductSchemeDetail fromDto(FormulaProductSchemeDetailDto dto) {
		return modelMapper.map(dto, FormulaProductSchemeDetail.class);
	}


}
