package com.sw.mes.production.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.production.dto.FormulaProductSchemeWeightDetailDto;
import com.sw.mes.production.entity.FormulaProductSchemeWeightDetail;
import com.sw.mes.production.service.FormulaProductSchemeWeightDetailService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/productSchemeWeightDetail")
@Slf4j
@Api(description = "产品配方重量明细接口")
public class FormulaProductSchemeWeightDetailController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private FormulaProductSchemeWeightDetailService formulaProductSchemeWeightDetailService;

	@ApiOperation(value = "分页查询产品配方重量明细列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<FormulaProductSchemeWeightDetailDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<FormulaProductSchemeWeightDetail> specification = DynamicSpecifications.bySearchFilter(request, FormulaProductSchemeWeightDetail.class);
    	Page<FormulaProductSchemeWeightDetail> page = formulaProductSchemeWeightDetailService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<FormulaProductSchemeWeightDetailDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有产品配方重量明细")
	@GetMapping("/listAll")
	public Result<List<FormulaProductSchemeWeightDetailDto>> listAll(ServletRequest request) {
		Specification<FormulaProductSchemeWeightDetail> specification = DynamicSpecifications.bySearchFilter(request, FormulaProductSchemeWeightDetail.class);
		List<FormulaProductSchemeWeightDetail> formulaProductSchemeWeightDetails = formulaProductSchemeWeightDetailService.findByExampleSort(specification, null);
		List<FormulaProductSchemeWeightDetailDto> formulaProductSchemeWeightDetailDtos = formulaProductSchemeWeightDetails.stream()
					.map(formulaProductSchemeWeightDetail -> toLazyDto(formulaProductSchemeWeightDetail))
					.collect(Collectors.toList());
		return ResultGenerator.success(formulaProductSchemeWeightDetailDtos);
	}

	@ApiOperation(value = "产品配方重量明细详细信息")
	@GetMapping("/{id}")
	public Result<FormulaProductSchemeWeightDetailDto> get(@PathVariable @ApiParam(value="产品配方重量明细id") Long id) {
		FormulaProductSchemeWeightDetail formulaProductSchemeWeightDetail = formulaProductSchemeWeightDetailService.get(id);
		if (null == formulaProductSchemeWeightDetail)
			return ResultGenerator.fail("该产品配方重量明细不存在");
		return ResultGenerator.success(toEagerDto(formulaProductSchemeWeightDetail));
	}

	@ApiOperation(value = "创建产品配方重量明细")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "产品配方重量明细对象") FormulaProductSchemeWeightDetailDto formulaProductSchemeWeightDetailDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		FormulaProductSchemeWeightDetail formulaProductSchemeWeightDetail = fromDto(formulaProductSchemeWeightDetailDto);
		formulaProductSchemeWeightDetailService.saveOrUpdate(formulaProductSchemeWeightDetail);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新产品配方重量明细")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "产品配方重量明细") FormulaProductSchemeWeightDetailDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            FormulaProductSchemeWeightDetail entity = formulaProductSchemeWeightDetailService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            FormulaProductSchemeWeightDetail entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			formulaProductSchemeWeightDetailService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除产品配方重量明细")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				formulaProductSchemeWeightDetailService.delete(id);
		}
		return ResultGenerator.success();
	}

	public FormulaProductSchemeWeightDetailDto toLazyDto(FormulaProductSchemeWeightDetail entity) {
		return modelMapper.map(entity, FormulaProductSchemeWeightDetailDto.class);
	}

	public FormulaProductSchemeWeightDetailDto toEagerDto(FormulaProductSchemeWeightDetail entity) {
		FormulaProductSchemeWeightDetailDto dto = modelMapper.map(entity, FormulaProductSchemeWeightDetailDto.class);
		return dto;
	}

	public FormulaProductSchemeWeightDetail fromDto(FormulaProductSchemeWeightDetailDto dto) {
		return modelMapper.map(dto, FormulaProductSchemeWeightDetail.class);
	}


}
