package com.sw.mes.sap.controller;


import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.sap.dto.ReportErpInputRecordDto;
import com.sw.mes.sap.entity.ReportErpInputRecord;
import com.sw.mes.sap.entity.ReportErpInputRecordDetail;
import com.sw.mes.sap.service.ReportErpInputRecordDetailService;
import com.sw.mes.sap.service.ReportErpInputRecordService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/ReportErpInputRecord")
@Slf4j
@Api(description = "拉取投料上报erp记录主表接口")
public class ReportErpInputRecordController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ReportErpInputRecordService reportErpInputRecordService;

	@Autowired
	private ReportErpInputRecordDetailService reportErpInputRecordDetailService;

	@ApiOperation(value = "分页查询拉取投料上报erp记录主表列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ReportErpInputRecordDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<ReportErpInputRecord> specification = DynamicSpecifications.bySearchFilter(request, ReportErpInputRecord.class);
    	Page<ReportErpInputRecord> page = reportErpInputRecordService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ReportErpInputRecordDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有拉取投料上报erp记录主表")
	@GetMapping("/listAll")
	public Result<List<ReportErpInputRecordDto>> listAll(ServletRequest request) {
		Specification<ReportErpInputRecord> specification = DynamicSpecifications.bySearchFilter(request, ReportErpInputRecord.class);
		List<ReportErpInputRecord> reportErpInputRecords = reportErpInputRecordService.findByExampleSort(specification, null);
		List<ReportErpInputRecordDto> reportErpInputRecordDtos = reportErpInputRecords.stream()
					.map(reportErpInputRecord -> toLazyDto(reportErpInputRecord))
					.collect(Collectors.toList());
		return ResultGenerator.success(reportErpInputRecordDtos);
	}

	@ApiOperation(value = "拉取投料上报erp记录主表详细信息")
	@GetMapping("/{id}")
	public Result<List<ReportErpInputRecordDetail>> get(@PathVariable @ApiParam(value="拉取投料上报erp记录主表id") Long id) {
		List<ReportErpInputRecordDetail> dtos = reportErpInputRecordDetailService.findRecordDetail(id);
		return ResultGenerator.success(dtos);
	}

	@ApiOperation(value = "创建拉取投料上报erp记录主表")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "拉取投料上报erp记录主表对象") ReportErpInputRecordDto reportErpInputRecordDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ReportErpInputRecord reportErpInputRecord = fromDto(reportErpInputRecordDto);
		reportErpInputRecordService.saveOrUpdate(reportErpInputRecord);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新拉取投料上报erp记录主表")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "拉取投料上报erp记录主表") ReportErpInputRecordDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ReportErpInputRecord entity = reportErpInputRecordService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ReportErpInputRecord entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			reportErpInputRecordService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除拉取投料上报erp记录主表")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				reportErpInputRecordService.delete(id);
		}
		return ResultGenerator.success();
	}

	@ApiOperation(value = "上报erp")
	@PostMapping("/pushToErp")
	public Result pushToErp(@RequestBody @ApiParam(value="拉取投料上报erp记录主表") ReportErpInputRecord dto) {
		try {
			reportErpInputRecordService.pushToErp(dto);
		} catch (Exception e) {
			e.printStackTrace();
			return ResultGenerator.fail(e.getMessage());
		}
		return ResultGenerator.success();
	}

	public ReportErpInputRecordDto toLazyDto(ReportErpInputRecord entity) {
		return modelMapper.map(entity, ReportErpInputRecordDto.class);
	}

	public ReportErpInputRecordDto toEagerDto(ReportErpInputRecord entity) {
		ReportErpInputRecordDto dto = modelMapper.map(entity, ReportErpInputRecordDto.class);
		return dto;
	}

	public ReportErpInputRecord fromDto(ReportErpInputRecordDto dto) {
		return modelMapper.map(dto, ReportErpInputRecord.class);
	}


}
