package com.sw.mes.sap.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.sap.dto.ReportErpOutputRecordDetailDto;
import com.sw.mes.sap.entity.ReportErpOutputRecordDetail;
import com.sw.mes.sap.service.ReportErpOutputRecordDetailService;
import com.sw.mes.yb.production.service.LongCrystalService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/ReportErpOutputRecordDetail")
@Slf4j
@Api(description = "拉取长晶棒产出信息上报erp记录子表接口")
public class ReportErpOutputRecordDetailController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ReportErpOutputRecordDetailService reportErpOutputRecordDetailService;

	@Autowired
	private LongCrystalService longCrystalService;

	@ApiOperation(value = "分页查询拉取长晶棒产出信息上报erp记录子表列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ReportErpOutputRecordDetailDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<ReportErpOutputRecordDetail> specification = DynamicSpecifications.bySearchFilter(request, ReportErpOutputRecordDetail.class);
    	Page<ReportErpOutputRecordDetail> page = reportErpOutputRecordDetailService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ReportErpOutputRecordDetailDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有拉取长晶棒产出信息上报erp记录子表")
	@GetMapping("/listAll")
	public Result<List<ReportErpOutputRecordDetailDto>> listAll(ServletRequest request) {
		Specification<ReportErpOutputRecordDetail> specification = DynamicSpecifications.bySearchFilter(request, ReportErpOutputRecordDetail.class);
		List<ReportErpOutputRecordDetail> reportErpOutputRecordDetails = reportErpOutputRecordDetailService.findByExampleSort(specification, null);
		List<ReportErpOutputRecordDetailDto> reportErpOutputRecordDetailDtos = reportErpOutputRecordDetails.stream()
					.map(reportErpOutputRecordDetail -> toLazyDto(reportErpOutputRecordDetail))
					.collect(Collectors.toList());
		return ResultGenerator.success(reportErpOutputRecordDetailDtos);
	}

	@ApiOperation(value = "拉取长晶棒产出信息上报erp记录子表详细信息")
	@GetMapping("/{id}")
	public Result<List<ReportErpOutputRecordDetailDto>> get(@PathVariable @ApiParam(value="拉取长晶棒产出信息上报erp记录子表id") Long id) {
		List<ReportErpOutputRecordDetail> list = reportErpOutputRecordDetailService.findByRid(id);
		if(CollectionUtils.isEmpty(list)){
			return ResultGenerator.fail("该拉取长晶棒产出信息上报erp记录子表不存在");
		}
		List<ReportErpOutputRecordDetailDto> reportErpOutputRecordDetailDtos = list.stream()
				.map(reportErpOutputRecordDetail -> toLazyDto(reportErpOutputRecordDetail))
				.collect(Collectors.toList());
		return ResultGenerator.success(reportErpOutputRecordDetailDtos);
	}

	@ApiOperation(value = "创建拉取长晶棒产出信息上报erp记录子表")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "拉取长晶棒产出信息上报erp记录子表对象") ReportErpOutputRecordDetailDto reportErpOutputRecordDetailDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ReportErpOutputRecordDetail reportErpOutputRecordDetail = fromDto(reportErpOutputRecordDetailDto);
		reportErpOutputRecordDetailService.saveOrUpdate(reportErpOutputRecordDetail);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新拉取长晶棒产出信息上报erp记录子表")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "拉取长晶棒产出信息上报erp记录子表") ReportErpOutputRecordDetailDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ReportErpOutputRecordDetail entity = reportErpOutputRecordDetailService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ReportErpOutputRecordDetail entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			reportErpOutputRecordDetailService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除拉取长晶棒产出信息上报erp记录子表")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
			reportErpOutputRecordDetailService.delete(id);
		}
		return ResultGenerator.success();
	}

	@ApiOperation(value = "长晶棒产出信息冲销")
	@PostMapping("/writeOffAccounts")
	public Result writeOffAccounts(@RequestBody @Valid @ApiParam(value = "拉取长晶棒产出信息冲销") ReportErpOutputRecordDetailDto dto) {

		Result result = reportErpOutputRecordDetailService.writeOffAccountsMethod(dto);
		return result;
	}

	public ReportErpOutputRecordDetailDto toLazyDto(ReportErpOutputRecordDetail entity) {
		return modelMapper.map(entity, ReportErpOutputRecordDetailDto.class);
	}

	public ReportErpOutputRecordDetailDto toEagerDto(ReportErpOutputRecordDetail entity) {
		ReportErpOutputRecordDetailDto dto = modelMapper.map(entity, ReportErpOutputRecordDetailDto.class);
		return dto;
	}

	public ReportErpOutputRecordDetail fromDto(ReportErpOutputRecordDetailDto dto) {
		return modelMapper.map(dto, ReportErpOutputRecordDetail.class);
	}


}
