package com.sw.mes.wms.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.wms.dto.ReportWmsRecordDto;
import com.sw.mes.wms.entity.ReportWmsRecord;
import com.sw.mes.wms.service.ReportWmsRecordService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/reportWmsRecord")
@Slf4j
@Api(description = "报工->wms记录表接口")
public class ReportWmsRecordController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ReportWmsRecordService reportWmsRecordService;

	@ApiOperation(value = "分页查询报工->wms记录表列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ReportWmsRecordDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<ReportWmsRecord> specification = DynamicSpecifications.bySearchFilter(request, ReportWmsRecord.class);
    	Page<ReportWmsRecord> page = reportWmsRecordService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ReportWmsRecordDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有报工->wms记录表")
	@GetMapping("/listAll")
	public Result<List<ReportWmsRecordDto>> listAll(ServletRequest request) {
		Specification<ReportWmsRecord> specification = DynamicSpecifications.bySearchFilter(request, ReportWmsRecord.class);
		List<ReportWmsRecord> reportWmsRecords = reportWmsRecordService.findByExampleSort(specification, null);
		List<ReportWmsRecordDto> reportWmsRecordDtos = reportWmsRecords.stream()
					.map(reportWmsRecord -> toLazyDto(reportWmsRecord))
					.collect(Collectors.toList());
		return ResultGenerator.success(reportWmsRecordDtos);
	}

	@ApiOperation(value = "报工->wms记录表详细信息")
	@GetMapping("/{id}")
	public Result<ReportWmsRecordDto> get(@PathVariable @ApiParam(value="报工->wms记录表id") Integer id) {
		ReportWmsRecord reportWmsRecord = reportWmsRecordService.get(id);
		if (null == reportWmsRecord)
			return ResultGenerator.fail("该报工->wms记录表不存在");
		return ResultGenerator.success(toEagerDto(reportWmsRecord));
	}

	@ApiOperation(value = "创建报工->wms记录表")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "报工->wms记录表对象") ReportWmsRecordDto reportWmsRecordDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ReportWmsRecord reportWmsRecord = fromDto(reportWmsRecordDto);
		reportWmsRecordService.saveOrUpdate(reportWmsRecord);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新报工->wms记录表")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "报工->wms记录表") ReportWmsRecordDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ReportWmsRecord entity = reportWmsRecordService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ReportWmsRecord entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			reportWmsRecordService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除报工->wms记录表")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				reportWmsRecordService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ReportWmsRecordDto toLazyDto(ReportWmsRecord entity) {
		return modelMapper.map(entity, ReportWmsRecordDto.class);
	}

	public ReportWmsRecordDto toEagerDto(ReportWmsRecord entity) {
		ReportWmsRecordDto dto = modelMapper.map(entity, ReportWmsRecordDto.class);
		return dto;
	}

	public ReportWmsRecord fromDto(ReportWmsRecordDto dto) {
		return modelMapper.map(dto, ReportWmsRecord.class);
	}


}
