package com.sw.mes.yb.accessory.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.accessory.dto.AccessoryDiscardDto;
import com.sw.mes.yb.accessory.dto.AccessoryDiscardExamineDto;
import com.sw.mes.yb.accessory.entity.AccessoryDiscard;
import com.sw.mes.yb.accessory.service.AccessoryDiscardService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/accessoryDiscard")
@Slf4j
@Api(description = "圆棒热场报废记录接口")
public class AccessoryDiscardController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private AccessoryDiscardService accessoryDiscardService;

	@ApiOperation(value = "分页查询圆棒热场报废记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<AccessoryDiscardDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<AccessoryDiscard> specification = DynamicSpecifications.bySearchFilter(request, AccessoryDiscard.class);
    	Page<AccessoryDiscard> page = accessoryDiscardService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<AccessoryDiscardDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有圆棒热场报废记录")
	@GetMapping("/listAll")
	public Result<List<AccessoryDiscardDto>> listAll(ServletRequest request) {
		Specification<AccessoryDiscard> specification = DynamicSpecifications.bySearchFilter(request, AccessoryDiscard.class);
		List<AccessoryDiscard> accessoryDiscards = accessoryDiscardService.findByExampleSort(specification, null);
		List<AccessoryDiscardDto> accessoryDiscardDtos = accessoryDiscards.stream()
					.map(accessoryDiscard -> toLazyDto(accessoryDiscard))
					.collect(Collectors.toList());
		return ResultGenerator.success(accessoryDiscardDtos);
	}

	@ApiOperation(value = "圆棒热场报废记录详细信息")
	@GetMapping("/{id}")
	public Result<AccessoryDiscardDto> get(@PathVariable @ApiParam(value="圆棒热场报废记录id") Long id) {
		AccessoryDiscard accessoryDiscard = accessoryDiscardService.get(id);
		if (null == accessoryDiscard)
			return ResultGenerator.fail("该圆棒热场报废记录不存在");
		return ResultGenerator.success(toEagerDto(accessoryDiscard));
	}

	@ApiOperation(value = "创建圆棒热场报废记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "圆棒热场报废记录对象") AccessoryDiscardDto accessoryDiscardDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		AccessoryDiscard accessoryDiscard = fromDto(accessoryDiscardDto);
		accessoryDiscardService.saveOrUpdate(accessoryDiscard);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "审核圆棒热场报废记录")
    @PostMapping("/examine")
    public Result examine(@RequestBody @Valid @ApiParam(value = "圆棒热场报废记录审核对象") AccessoryDiscardExamineDto examineDto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        accessoryDiscardService.examine(examineDto);
        return ResultGenerator.success();
    }

    @ApiOperation(value = "更新圆棒热场报废记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "圆棒热场报废记录") AccessoryDiscardDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            AccessoryDiscard entity = accessoryDiscardService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            AccessoryDiscard entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			accessoryDiscardService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除圆棒热场报废记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				accessoryDiscardService.delete(id);
		}
		return ResultGenerator.success();
	}

	public AccessoryDiscardDto toLazyDto(AccessoryDiscard entity) {
		return modelMapper.map(entity, AccessoryDiscardDto.class);
	}

	public AccessoryDiscardDto toEagerDto(AccessoryDiscard entity) {
		AccessoryDiscardDto dto = modelMapper.map(entity, AccessoryDiscardDto.class);
		return dto;
	}

	public AccessoryDiscard fromDto(AccessoryDiscardDto dto) {
		return modelMapper.map(dto, AccessoryDiscard.class);
	}


}
