package com.sw.mes.yb.accessory.controller;


import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.accessory.dto.AccessoryLogDto;
import com.sw.mes.yb.accessory.entity.AccessoryLog;
import com.sw.mes.yb.accessory.service.AccessoryLogService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/accessoryLog")
@Slf4j
@Api(description = "热场接口调用日志接口")
public class AccessoryLogController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private AccessoryLogService accessoryLogService;

	@ApiOperation(value = "分页查询热场接口调用日志列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<AccessoryLogDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<AccessoryLog> specification = DynamicSpecifications.bySearchFilter(request, AccessoryLog.class);
    	Page<AccessoryLog> page = accessoryLogService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<AccessoryLogDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有热场接口调用日志")
	@GetMapping("/listAll")
	public Result<List<AccessoryLogDto>> listAll(ServletRequest request) {
		Specification<AccessoryLog> specification = DynamicSpecifications.bySearchFilter(request, AccessoryLog.class);
		List<AccessoryLog> accessoryLogs = accessoryLogService.findByExampleSort(specification, null);
		List<AccessoryLogDto> accessoryLogDtos = accessoryLogs.stream()
					.map(accessoryLog -> toLazyDto(accessoryLog))
					.collect(Collectors.toList());
		return ResultGenerator.success(accessoryLogDtos);
	}

	@ApiOperation(value = "热场接口调用日志详细信息")
	@GetMapping("/{id}")
	public Result<AccessoryLogDto> get(@PathVariable @ApiParam(value="热场接口调用日志id") Integer id) {
		AccessoryLog accessoryLog = accessoryLogService.get(id);
		if (null == accessoryLog)
			return ResultGenerator.fail("该热场接口调用日志不存在");
		return ResultGenerator.success(toEagerDto(accessoryLog));
	}

	@ApiOperation(value = "创建热场接口调用日志")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "热场接口调用日志对象") AccessoryLogDto accessoryLogDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		AccessoryLog accessoryLog = fromDto(accessoryLogDto);
		accessoryLogService.saveOrUpdate(accessoryLog);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新热场接口调用日志")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "热场接口调用日志") AccessoryLogDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            AccessoryLog entity = accessoryLogService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            AccessoryLog entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			accessoryLogService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除热场接口调用日志")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				accessoryLogService.delete(id);
		}
		return ResultGenerator.success();
	}

	public AccessoryLogDto toLazyDto(AccessoryLog entity) {
		return modelMapper.map(entity, AccessoryLogDto.class);
	}

	public AccessoryLogDto toEagerDto(AccessoryLog entity) {
		AccessoryLogDto dto = modelMapper.map(entity, AccessoryLogDto.class);
		return dto;
	}

	public AccessoryLog fromDto(AccessoryLogDto dto) {
		return modelMapper.map(dto, AccessoryLog.class);
	}


}
