package com.sw.mes.yb.accessory.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.accessory.dto.EquimentStatusDto;
import com.sw.mes.yb.accessory.dto.EquipmentDeviceStatusDto;
import com.sw.mes.yb.accessory.entity.EquipmentDeviceStatus;
import com.sw.mes.yb.accessory.service.EquipmentDeviceStatusService;
import com.sw.mes.yb.accessory.vo.EapToMesDeviceStatusVo;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/equipmentDeviceStatus")
@Slf4j
@Api(description = "设备状态接口")
public class EquipmentDeviceStatusController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private EquipmentDeviceStatusService equipmentDeviceStatusService;

	@ApiOperation(value = "分页查询设备状态列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<EquipmentDeviceStatusDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<EquipmentDeviceStatus> specification = DynamicSpecifications.bySearchFilter(request, EquipmentDeviceStatus.class);
    	Page<EquipmentDeviceStatus> page = equipmentDeviceStatusService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<EquipmentDeviceStatusDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有设备状态")
	@GetMapping("/listAll")
	public Result<List<EquipmentDeviceStatusDto>> listAll(ServletRequest request) {
		Specification<EquipmentDeviceStatus> specification = DynamicSpecifications.bySearchFilter(request, EquipmentDeviceStatus.class);
		List<EquipmentDeviceStatus> equipmentDeviceStatuss = equipmentDeviceStatusService.findByExampleSort(specification, null);
		List<EquipmentDeviceStatusDto> equipmentDeviceStatusDtos = equipmentDeviceStatuss.stream()
					.map(equipmentDeviceStatus -> toLazyDto(equipmentDeviceStatus))
					.collect(Collectors.toList());
		return ResultGenerator.success(equipmentDeviceStatusDtos);
	}

	@ApiOperation(value = "设备状态详细信息")
	@GetMapping("/{id}")
	public Result<EquipmentDeviceStatusDto> get(@PathVariable @ApiParam(value="设备状态id") Long id) {
		EquipmentDeviceStatus equipmentDeviceStatus = equipmentDeviceStatusService.get(id);
		if (null == equipmentDeviceStatus) {
            return ResultGenerator.fail("该设备状态不存在");
        }
		return ResultGenerator.success(toEagerDto(equipmentDeviceStatus));
	}

	@ApiOperation(value = "创建设备状态")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "设备状态对象") EapToMesDeviceStatusVo vo, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		equipmentDeviceStatusService.createDeviceStatus(vo);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新设备状态")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "设备状态") EquipmentDeviceStatusDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if (dto.getId() == null) {
            return ResultGenerator.fail("id不能为空");
        }
            EquipmentDeviceStatus entity = equipmentDeviceStatusService.get(dto.getId());
        if (entity == null) {
            return ResultGenerator.fail("原信息不存在");
        }
        try {
            EquipmentDeviceStatus entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			equipmentDeviceStatusService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除设备状态")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				equipmentDeviceStatusService.delete(id);
		}
		return ResultGenerator.success();
	}

	public EquipmentDeviceStatusDto toLazyDto(EquipmentDeviceStatus entity) {
		return modelMapper.map(entity, EquipmentDeviceStatusDto.class);
	}

	public EquipmentDeviceStatusDto toEagerDto(EquipmentDeviceStatus entity) {
		EquipmentDeviceStatusDto dto = modelMapper.map(entity, EquipmentDeviceStatusDto.class);
		return dto;
	}

	public EquipmentDeviceStatus fromDto(EquipmentDeviceStatusDto dto) {
		return modelMapper.map(dto, EquipmentDeviceStatus.class);
	}


}
