package com.sw.mes.yb.alloy.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.alloy.dto.CorrectingRecordDto;
import com.sw.mes.yb.alloy.entity.CorrectingRecord;
import com.sw.mes.yb.alloy.service.CorrectingRecordService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;



/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/correctingRecord")
@Slf4j
@Api(description = "修正记录接口")
public class CorrectingRecordController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private CorrectingRecordService correctingRecordService;

	@ApiOperation(value = "分页查询修正记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<CorrectingRecordDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<CorrectingRecord> specification = DynamicSpecifications.bySearchFilter(request, CorrectingRecord.class);
    	Page<CorrectingRecord> page = correctingRecordService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<CorrectingRecordDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有修正记录")
	@GetMapping("/listAll")
	public Result<List<CorrectingRecordDto>> listAll(ServletRequest request) {
		Specification<CorrectingRecord> specification = DynamicSpecifications.bySearchFilter(request, CorrectingRecord.class);
		List<CorrectingRecord> correctingRecords = correctingRecordService.findByExampleSort(specification, null);
		List<CorrectingRecordDto> correctingRecordDtos = correctingRecords.stream()
					.map(correctingRecord -> toLazyDto(correctingRecord))
					.collect(Collectors.toList());
		return ResultGenerator.success(correctingRecordDtos);
	}

	@ApiOperation(value = "修正记录详细信息")
	@GetMapping("/{id}")
	public Result<CorrectingRecordDto> get(@PathVariable @ApiParam(value="修正记录id") Integer id) {
		CorrectingRecord correctingRecord = correctingRecordService.get(id);
		if (null == correctingRecord)
			return ResultGenerator.fail("该修正记录不存在");
		return ResultGenerator.success(toEagerDto(correctingRecord));
	}

	@ApiOperation(value = "创建修正记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "修正记录对象") CorrectingRecordDto correctingRecordDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		CorrectingRecord correctingRecord = fromDto(correctingRecordDto);
		correctingRecordService.saveOrUpdate(correctingRecord);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新修正记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "修正记录") CorrectingRecordDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            CorrectingRecord entity = correctingRecordService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            CorrectingRecord entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			correctingRecordService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除修正记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				correctingRecordService.delete(id);
		}
		return ResultGenerator.success();
	}

	public CorrectingRecordDto toLazyDto(CorrectingRecord entity) {
		return modelMapper.map(entity, CorrectingRecordDto.class);
	}

	public CorrectingRecordDto toEagerDto(CorrectingRecord entity) {
		CorrectingRecordDto dto = modelMapper.map(entity, CorrectingRecordDto.class);
		return dto;
	}

	public CorrectingRecord fromDto(CorrectingRecordDto dto) {
		return modelMapper.map(dto, CorrectingRecord.class);
	}


}
