package com.sw.mes.yb.alloy.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.alloy.dto.InspectionListDto;
import com.sw.mes.yb.alloy.entity.InspectionList;
import com.sw.mes.yb.alloy.service.InspectionListService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/inspectionList")
@Slf4j
@Api(description = "检验列表接口")
public class InspectionListController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private InspectionListService inspectionListService;

	@ApiOperation(value = "分页查询检验列表列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<InspectionListDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<InspectionList> specification = DynamicSpecifications.bySearchFilter(request, InspectionList.class);
    	Page<InspectionList> page = inspectionListService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<InspectionListDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有检验列表")
	@GetMapping("/listAll")
	public Result<List<InspectionListDto>> listAll(ServletRequest request) {
		Specification<InspectionList> specification = DynamicSpecifications.bySearchFilter(request, InspectionList.class);
		List<InspectionList> inspectionLists = inspectionListService.findByExampleSort(specification, null);
		List<InspectionListDto> inspectionListDtos = inspectionLists.stream()
					.map(inspectionList -> toLazyDto(inspectionList))
					.collect(Collectors.toList());
		return ResultGenerator.success(inspectionListDtos);
	}

	@ApiOperation(value = "检验列表详细信息")
	@GetMapping("/{id}")
	public Result<InspectionListDto> get(@PathVariable @ApiParam(value="检验列表id") Integer id) {
		InspectionList inspectionList = inspectionListService.get(id);
		if (null == inspectionList)
			return ResultGenerator.fail("该检验列表不存在");
		return ResultGenerator.success(toEagerDto(inspectionList));
	}

	@ApiOperation(value = "创建检验列表")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "检验列表对象") InspectionListDto inspectionListDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		InspectionList inspectionList = fromDto(inspectionListDto);
		inspectionListService.saveOrUpdate(inspectionList);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新检验列表")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "检验列表") InspectionListDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            InspectionList entity = inspectionListService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            InspectionList entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			inspectionListService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除检验列表")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				inspectionListService.delete(id);
		}
		return ResultGenerator.success();
	}

	public InspectionListDto toLazyDto(InspectionList entity) {
		return modelMapper.map(entity, InspectionListDto.class);
	}

	public InspectionListDto toEagerDto(InspectionList entity) {
		InspectionListDto dto = modelMapper.map(entity, InspectionListDto.class);
		return dto;
	}

	public InspectionList fromDto(InspectionListDto dto) {
		return modelMapper.map(dto, InspectionList.class);
	}


}
