package com.sw.mes.yb.mcs.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.mcs.entity.RecycledMaterialCase;
import com.sw.mes.yb.mcs.dto.RecycledMaterialCaseDto;
import com.sw.mes.yb.mcs.service.RecycledMaterialCaseService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/recycledMaterialCase")
@Slf4j
@Api(description = "RecycledMaterialCase接口")
public class RecycledMaterialCaseController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private RecycledMaterialCaseService recycledMaterialCaseService;

	@ApiOperation(value = "分页查询RecycledMaterialCase列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<RecycledMaterialCaseDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<RecycledMaterialCase> specification = DynamicSpecifications.bySearchFilter(request, RecycledMaterialCase.class);
    	Page<RecycledMaterialCase> page = recycledMaterialCaseService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<RecycledMaterialCaseDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有RecycledMaterialCase")
	@GetMapping("/listAll")
	public Result<List<RecycledMaterialCaseDto>> listAll(ServletRequest request) {
		Specification<RecycledMaterialCase> specification = DynamicSpecifications.bySearchFilter(request, RecycledMaterialCase.class);
		List<RecycledMaterialCase> recycledMaterialCases = recycledMaterialCaseService.findByExampleSort(specification, null);
		List<RecycledMaterialCaseDto> recycledMaterialCaseDtos = recycledMaterialCases.stream()
					.map(recycledMaterialCase -> toLazyDto(recycledMaterialCase))
					.collect(Collectors.toList());
		return ResultGenerator.success(recycledMaterialCaseDtos);
	}

	@ApiOperation(value = "RecycledMaterialCase详细信息")
	@GetMapping("/{id}")
	public Result<RecycledMaterialCaseDto> get(@PathVariable @ApiParam(value="RecycledMaterialCaseid") Long id) {
		RecycledMaterialCase recycledMaterialCase = recycledMaterialCaseService.get(id);
		if (null == recycledMaterialCase)
			return ResultGenerator.fail("该RecycledMaterialCase不存在");
		return ResultGenerator.success(toEagerDto(recycledMaterialCase));
	}

	@ApiOperation(value = "创建RecycledMaterialCase")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "RecycledMaterialCase对象") RecycledMaterialCaseDto recycledMaterialCaseDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		RecycledMaterialCase recycledMaterialCase = fromDto(recycledMaterialCaseDto);
		recycledMaterialCaseService.saveOrUpdate(recycledMaterialCase);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新RecycledMaterialCase")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "RecycledMaterialCase") RecycledMaterialCaseDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            RecycledMaterialCase entity = recycledMaterialCaseService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            RecycledMaterialCase entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			recycledMaterialCaseService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除RecycledMaterialCase")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				recycledMaterialCaseService.delete(id);
		}
		return ResultGenerator.success();
	}

	public RecycledMaterialCaseDto toLazyDto(RecycledMaterialCase entity) {
		return modelMapper.map(entity, RecycledMaterialCaseDto.class);
	}

	public RecycledMaterialCaseDto toEagerDto(RecycledMaterialCase entity) {
		RecycledMaterialCaseDto dto = modelMapper.map(entity, RecycledMaterialCaseDto.class);
		return dto;
	}

	public RecycledMaterialCase fromDto(RecycledMaterialCaseDto dto) {
		return modelMapper.map(dto, RecycledMaterialCase.class);
	}


}
