package com.sw.mes.yb.mcs.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.mcs.entity.SegmentInspectionPassPointRecord;
import com.sw.mes.yb.mcs.dto.SegmentInspectionPassPointRecordDto;
import com.sw.mes.yb.mcs.service.SegmentInspectionPassPointRecordService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/segmentInspectionPassPointRecord")
@Slf4j
@Api(description = "SegmentInspectionPassPointRecord接口")
public class SegmentInspectionPassPointRecordController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private SegmentInspectionPassPointRecordService segmentInspectionPassPointRecordService;

	@ApiOperation(value = "分页查询SegmentInspectionPassPointRecord列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<SegmentInspectionPassPointRecordDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<SegmentInspectionPassPointRecord> specification = DynamicSpecifications.bySearchFilter(request, SegmentInspectionPassPointRecord.class);
    	Page<SegmentInspectionPassPointRecord> page = segmentInspectionPassPointRecordService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<SegmentInspectionPassPointRecordDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有SegmentInspectionPassPointRecord")
	@GetMapping("/listAll")
	public Result<List<SegmentInspectionPassPointRecordDto>> listAll(ServletRequest request) {
		Specification<SegmentInspectionPassPointRecord> specification = DynamicSpecifications.bySearchFilter(request, SegmentInspectionPassPointRecord.class);
		List<SegmentInspectionPassPointRecord> segmentInspectionPassPointRecords = segmentInspectionPassPointRecordService.findByExampleSort(specification, null);
		List<SegmentInspectionPassPointRecordDto> segmentInspectionPassPointRecordDtos = segmentInspectionPassPointRecords.stream()
					.map(segmentInspectionPassPointRecord -> toLazyDto(segmentInspectionPassPointRecord))
					.collect(Collectors.toList());
		return ResultGenerator.success(segmentInspectionPassPointRecordDtos);
	}

	@ApiOperation(value = "SegmentInspectionPassPointRecord详细信息")
	@GetMapping("/{id}")
	public Result<SegmentInspectionPassPointRecordDto> get(@PathVariable @ApiParam(value="SegmentInspectionPassPointRecordid") Long id) {
		SegmentInspectionPassPointRecord segmentInspectionPassPointRecord = segmentInspectionPassPointRecordService.get(id);
		if (null == segmentInspectionPassPointRecord)
			return ResultGenerator.fail("该SegmentInspectionPassPointRecord不存在");
		return ResultGenerator.success(toEagerDto(segmentInspectionPassPointRecord));
	}

	@ApiOperation(value = "创建SegmentInspectionPassPointRecord")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "SegmentInspectionPassPointRecord对象") SegmentInspectionPassPointRecordDto segmentInspectionPassPointRecordDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		SegmentInspectionPassPointRecord segmentInspectionPassPointRecord = fromDto(segmentInspectionPassPointRecordDto);
		segmentInspectionPassPointRecordService.saveOrUpdate(segmentInspectionPassPointRecord);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新SegmentInspectionPassPointRecord")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "SegmentInspectionPassPointRecord") SegmentInspectionPassPointRecordDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            SegmentInspectionPassPointRecord entity = segmentInspectionPassPointRecordService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            SegmentInspectionPassPointRecord entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			segmentInspectionPassPointRecordService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除SegmentInspectionPassPointRecord")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				segmentInspectionPassPointRecordService.delete(id);
		}
		return ResultGenerator.success();
	}

	public SegmentInspectionPassPointRecordDto toLazyDto(SegmentInspectionPassPointRecord entity) {
		return modelMapper.map(entity, SegmentInspectionPassPointRecordDto.class);
	}

	public SegmentInspectionPassPointRecordDto toEagerDto(SegmentInspectionPassPointRecord entity) {
		SegmentInspectionPassPointRecordDto dto = modelMapper.map(entity, SegmentInspectionPassPointRecordDto.class);
		return dto;
	}

	public SegmentInspectionPassPointRecord fromDto(SegmentInspectionPassPointRecordDto dto) {
		return modelMapper.map(dto, SegmentInspectionPassPointRecord.class);
	}


}
