package com.sw.mes.yb.pad.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.pad.entity.RecutDetail;
import com.sw.mes.yb.pad.dto.RecutDetailDto;
import com.sw.mes.yb.pad.service.RecutDetailService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/RecutDetail")
@Slf4j
@Api(description = "方棒圆棒追溯接口")
public class RecutDetailController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private RecutDetailService recutDetailService;

	@ApiOperation(value = "分页查询方棒圆棒追溯列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<RecutDetailDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<RecutDetail> specification = DynamicSpecifications.bySearchFilter(request, RecutDetail.class);
    	Page<RecutDetail> page = recutDetailService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<RecutDetailDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有方棒圆棒追溯")
	@GetMapping("/listAll")
	public Result<List<RecutDetailDto>> listAll(ServletRequest request) {
		Specification<RecutDetail> specification = DynamicSpecifications.bySearchFilter(request, RecutDetail.class);
		List<RecutDetail> recutDetails = recutDetailService.findByExampleSort(specification, null);
		List<RecutDetailDto> recutDetailDtos = recutDetails.stream()
					.map(recutDetail -> toLazyDto(recutDetail))
					.collect(Collectors.toList());
		return ResultGenerator.success(recutDetailDtos);
	}

	@ApiOperation(value = "方棒圆棒追溯详细信息")
	@GetMapping("/{id}")
	public Result<RecutDetailDto> get(@PathVariable @ApiParam(value="RecutDetailid") Long id) {
		RecutDetail recutDetail = recutDetailService.get(id);
		if (null == recutDetail) {
            return ResultGenerator.fail("该RecutDetail不存在");
        }
		return ResultGenerator.success(toEagerDto(recutDetail));
	}

	@ApiOperation(value = "创建方棒圆棒追溯")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "RecutDetail对象") RecutDetailDto recutDetailDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		RecutDetail recutDetail = fromDto(recutDetailDto);
		recutDetailService.saveOrUpdate(recutDetail);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新方棒圆棒追溯")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "RecutDetail") RecutDetailDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if (dto.getId() == null) {
            return ResultGenerator.fail("id不能为空");
        }
            RecutDetail entity = recutDetailService.get(dto.getId());
        if (entity == null) {
            return ResultGenerator.fail("原信息不存在");
        }
        try {
            RecutDetail entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			recutDetailService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除方棒圆棒追溯")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				recutDetailService.delete(id);
		}
		return ResultGenerator.success();
	}

	public RecutDetailDto toLazyDto(RecutDetail entity) {
		return modelMapper.map(entity, RecutDetailDto.class);
	}

	public RecutDetailDto toEagerDto(RecutDetail entity) {
		RecutDetailDto dto = modelMapper.map(entity, RecutDetailDto.class);
		return dto;
	}

	public RecutDetail fromDto(RecutDetailDto dto) {
		return modelMapper.map(dto, RecutDetail.class);
	}


}
