package com.sw.mes.yb.plan.controller;

import com.sw.mes.core.annotation.Jinko;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.plan.dto.ProductionScheduleLogDto;
import com.sw.mes.yb.plan.entity.ProductionScheduleLog;
import com.sw.mes.yb.plan.service.ProductionScheduleLogService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

@Jinko(value = "炉内转订单记录", author = "wxl")
@RestController
@RequestMapping("/productionScheduleLog")
@Slf4j
@Api(description = "圆棒工单排程记录接口")
public class ProductionScheduleLogController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ProductionScheduleLogService productionScheduleLogService;

	@ApiOperation(value = "分页查询圆棒工单排程记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ProductionScheduleLogDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<ProductionScheduleLog> specification = DynamicSpecifications.bySearchFilter(request, ProductionScheduleLog.class);
    	Page<ProductionScheduleLog> page = productionScheduleLogService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ProductionScheduleLogDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有圆棒工单排程记录")
	@GetMapping("/listAll")
	public Result<List<ProductionScheduleLogDto>> listAll(ServletRequest request) {
		Specification<ProductionScheduleLog> specification = DynamicSpecifications.bySearchFilter(request, ProductionScheduleLog.class);
		List<ProductionScheduleLog> productionScheduleLogs = productionScheduleLogService.findByExampleSort(specification, null);
		List<ProductionScheduleLogDto> productionScheduleLogDtos = productionScheduleLogs.stream()
					.map(productionScheduleLog -> toLazyDto(productionScheduleLog))
					.collect(Collectors.toList());
		return ResultGenerator.success(productionScheduleLogDtos);
	}

	@ApiOperation(value = "圆棒工单排程记录详细信息")
	@GetMapping("/{id}")
	public Result<ProductionScheduleLogDto> get(@PathVariable @ApiParam(value="圆棒工单排程记录id") Long id) {
		ProductionScheduleLog productionScheduleLog = productionScheduleLogService.get(id);
		if (null == productionScheduleLog)
			return ResultGenerator.fail("该圆棒工单排程记录不存在");
		return ResultGenerator.success(toEagerDto(productionScheduleLog));
	}

	@ApiOperation(value = "创建圆棒工单排程记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "圆棒工单排程记录对象") ProductionScheduleLogDto productionScheduleLogDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ProductionScheduleLog productionScheduleLog = fromDto(productionScheduleLogDto);
		productionScheduleLogService.saveOrUpdate(productionScheduleLog);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新圆棒工单排程记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "圆棒工单排程记录") ProductionScheduleLogDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ProductionScheduleLog entity = productionScheduleLogService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ProductionScheduleLog entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			productionScheduleLogService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除圆棒工单排程记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				productionScheduleLogService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ProductionScheduleLogDto toLazyDto(ProductionScheduleLog entity) {
		return modelMapper.map(entity, ProductionScheduleLogDto.class);
	}

	public ProductionScheduleLogDto toEagerDto(ProductionScheduleLog entity) {
		ProductionScheduleLogDto dto = modelMapper.map(entity, ProductionScheduleLogDto.class);
		return dto;
	}

	public ProductionScheduleLog fromDto(ProductionScheduleLogDto dto) {
		return modelMapper.map(dto, ProductionScheduleLog.class);
	}


}
