package com.sw.mes.yb.production.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.production.dto.AgainTaskPrepare.AgainTaskPrepareDto;
import com.sw.mes.yb.production.entity.AgainTaskPrepare.AgainTaskPrepare;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.production.service.AgainTaskPrepareService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/againTaskPrepare")
@Slf4j
@Api(description = "预装料任务接口")
public class AgainTaskPrepareController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private AgainTaskPrepareService againTaskPrepareService;

	@ApiOperation(value = "分页查询预装料任务列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<AgainTaskPrepareDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<AgainTaskPrepare> specification = DynamicSpecifications.bySearchFilter(request, AgainTaskPrepare.class);
    	Page<AgainTaskPrepare> page = againTaskPrepareService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<AgainTaskPrepareDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有预装料任务")
	@GetMapping("/listAll")
	public Result<List<AgainTaskPrepareDto>> listAll(ServletRequest request) {
		Specification<AgainTaskPrepare> specification = DynamicSpecifications.bySearchFilter(request, AgainTaskPrepare.class);
		List<AgainTaskPrepare> againTaskPrepares = againTaskPrepareService.findByExampleSort(specification, null);
		List<AgainTaskPrepareDto> againTaskPrepareDtos = againTaskPrepares.stream()
					.map(againTaskPrepare -> toLazyDto(againTaskPrepare))
					.collect(Collectors.toList());
		return ResultGenerator.success(againTaskPrepareDtos);
	}

	@ApiOperation(value = "预装料任务详细信息")
	@GetMapping("/{id}")
	public Result<AgainTaskPrepareDto> get(@PathVariable @ApiParam(value="预装料任务id") Long id) {
		AgainTaskPrepare againTaskPrepare = againTaskPrepareService.get(id);
		if (null == againTaskPrepare)
			return ResultGenerator.fail("该预装料任务不存在");
		return ResultGenerator.success(toEagerDto(againTaskPrepare));
	}

	@ApiOperation(value = "创建预装料任务")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "预装料任务对象") AgainTaskPrepareDto againTaskPrepareDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		AgainTaskPrepare againTaskPrepare = fromDto(againTaskPrepareDto);
		againTaskPrepareService.saveOrUpdate(againTaskPrepare);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新预装料任务")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "预装料任务") AgainTaskPrepareDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            AgainTaskPrepare entity = againTaskPrepareService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            AgainTaskPrepare entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			againTaskPrepareService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除预装料任务")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
		for (Long id : ids) {
				againTaskPrepareService.delete(id);
		}
		return ResultGenerator.success();
	}

	public AgainTaskPrepareDto toLazyDto(AgainTaskPrepare entity) {
		return modelMapper.map(entity, AgainTaskPrepareDto.class);
	}

	public AgainTaskPrepareDto toEagerDto(AgainTaskPrepare entity) {
		AgainTaskPrepareDto dto = modelMapper.map(entity, AgainTaskPrepareDto.class);
		return dto;
	}

	public AgainTaskPrepare fromDto(AgainTaskPrepareDto dto) {
		return modelMapper.map(dto, AgainTaskPrepare.class);
	}


}
