package com.sw.mes.yb.production.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.production.dto.AgainTaskPrepare.InputTaskDatailPrepareDto;
import com.sw.mes.yb.production.entity.AgainTaskPrepare.InputTaskDatailPrepare;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.production.service.InputTaskDatailPrepareService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/inputTaskDatailPrepare")
@Slf4j
@Api(description = "预装料灌料明细表接口")
public class InputTaskDatailPrepareController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private InputTaskDatailPrepareService inputTaskDatailPrepareService;

	@ApiOperation(value = "分页查询预装料灌料明细表列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<InputTaskDatailPrepareDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<InputTaskDatailPrepare> specification = DynamicSpecifications.bySearchFilter(request, InputTaskDatailPrepare.class);
    	Page<InputTaskDatailPrepare> page = inputTaskDatailPrepareService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<InputTaskDatailPrepareDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有预装料灌料明细表")
	@GetMapping("/listAll")
	public Result<List<InputTaskDatailPrepareDto>> listAll(ServletRequest request) {
		Specification<InputTaskDatailPrepare> specification = DynamicSpecifications.bySearchFilter(request, InputTaskDatailPrepare.class);
		List<InputTaskDatailPrepare> inputTaskDatailPrepares = inputTaskDatailPrepareService.findByExampleSort(specification, null);
		List<InputTaskDatailPrepareDto> inputTaskDatailPrepareDtos = inputTaskDatailPrepares.stream()
					.map(inputTaskDatailPrepare -> toLazyDto(inputTaskDatailPrepare))
					.collect(Collectors.toList());
		return ResultGenerator.success(inputTaskDatailPrepareDtos);
	}

	@ApiOperation(value = "预装料灌料明细表详细信息")
	@GetMapping("/{id}")
	public Result<InputTaskDatailPrepareDto> get(@PathVariable @ApiParam(value="预装料灌料明细表id") Long id) {
		InputTaskDatailPrepare inputTaskDatailPrepare = inputTaskDatailPrepareService.get(id);
		if (null == inputTaskDatailPrepare)
			return ResultGenerator.fail("该预装料灌料明细表不存在");
		return ResultGenerator.success(toEagerDto(inputTaskDatailPrepare));
	}

	@ApiOperation(value = "创建预装料灌料明细表")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "预装料灌料明细表对象") InputTaskDatailPrepareDto inputTaskDatailPrepareDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		InputTaskDatailPrepare inputTaskDatailPrepare = fromDto(inputTaskDatailPrepareDto);
		inputTaskDatailPrepareService.saveOrUpdate(inputTaskDatailPrepare);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新预装料灌料明细表")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "预装料灌料明细表") InputTaskDatailPrepareDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            InputTaskDatailPrepare entity = inputTaskDatailPrepareService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            InputTaskDatailPrepare entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			inputTaskDatailPrepareService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除预装料灌料明细表")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
		for (Long id : ids) {
				inputTaskDatailPrepareService.delete(id);
		}
		return ResultGenerator.success();
	}

	public InputTaskDatailPrepareDto toLazyDto(InputTaskDatailPrepare entity) {
		return modelMapper.map(entity, InputTaskDatailPrepareDto.class);
	}

	public InputTaskDatailPrepareDto toEagerDto(InputTaskDatailPrepare entity) {
		InputTaskDatailPrepareDto dto = modelMapper.map(entity, InputTaskDatailPrepareDto.class);
		return dto;
	}

	public InputTaskDatailPrepare fromDto(InputTaskDatailPrepareDto dto) {
		return modelMapper.map(dto, InputTaskDatailPrepare.class);
	}


}
