package com.sw.mes.yb.production.controller;

import com.sw.mes.core.dto.LongCrystalDto1;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.*;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.production.entity.OutputRecord;
import com.sw.mes.yb.production.dto.OutputRecordDto;
import com.sw.mes.yb.production.service.OutputRecordService;


/**
 * @Author
 * @Description：
 * @Date: Create in
 * @Modified By:
 */
@RestController
@RequestMapping("/OutputRecord")
@Slf4j
@Api(description = "产出未灌料记录接口")
public class OutputRecordController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private OutputRecordService outputRecordService;

	@ApiOperation(value = "分页查询产出未灌料记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<OutputRecordDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<OutputRecord> specification = DynamicSpecifications.bySearchFilter(request, OutputRecord.class);
    	Page<OutputRecord> page = outputRecordService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<OutputRecordDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有产出未灌料记录")
	@GetMapping("/listAll")
	public Result<List<OutputRecordDto>> listAll(ServletRequest request) {
		Specification<OutputRecord> specification = DynamicSpecifications.bySearchFilter(request, OutputRecord.class);
		List<OutputRecord> outputRecords = outputRecordService.findByExampleSort(specification, null);
		List<OutputRecordDto> outputRecordDtos = outputRecords.stream()
					.map(outputRecord -> toLazyDto(outputRecord))
					.collect(Collectors.toList());
		return ResultGenerator.success(outputRecordDtos);
	}

	@ApiOperation(value = "产出未灌料记录详细信息")
	@GetMapping("/{id}")
	public Result<OutputRecordDto> get(@PathVariable @ApiParam(value="产出未灌料记录id") Long id) {
		OutputRecord outputRecord = outputRecordService.get(id);
		if (null == outputRecord){
			return ResultGenerator.fail("该产出未灌料记录不存在");
		}
		return ResultGenerator.success(toEagerDto(outputRecord));
	}

	@ApiOperation(value = "创建产出未灌料记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "产出未灌料记录对象") OutputRecordDto outputRecordDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		OutputRecord outputRecord = fromDto(outputRecordDto);
		outputRecordService.saveOrUpdate(outputRecord);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新产出未灌料记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "产出未灌料记录") OutputRecordDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()){
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
        if (dto.getId() == null){
            return ResultGenerator.fail("id不能为空");
		}
		OutputRecord entity = outputRecordService.get(dto.getId());
        if (entity == null){
            return ResultGenerator.fail("原信息不存在");
		}
        try {
            OutputRecord entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			outputRecordService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除产出未灌料记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				outputRecordService.delete(id);
		}
		return ResultGenerator.success();
	}

	public OutputRecordDto toLazyDto(OutputRecord entity) {
		return modelMapper.map(entity, OutputRecordDto.class);
	}

	public OutputRecordDto toEagerDto(OutputRecord entity) {
		OutputRecordDto dto = modelMapper.map(entity, OutputRecordDto.class);
		return dto;
	}

	public OutputRecord fromDto(OutputRecordDto dto) {
		return modelMapper.map(dto, OutputRecord.class);
	}


}
