package com.sw.mes.yb.production.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.production.dto.ProductionFurnaceMaterialSplitDto;
import com.sw.mes.yb.production.entity.ProductionFurnaceMaterialSplit;
import com.sw.mes.yb.production.service.ProductionFurnaceMaterialSplitService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


@RestController
@RequestMapping("/furnaceMaterialSplit")
@Slf4j
@Api(description = "投料分解记录接口")
public class ProductionFurnaceMaterialSplitController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ProductionFurnaceMaterialSplitService productionFurnaceMaterialSplitService;

	@ApiOperation(value = "分页查询投料分解记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ProductionFurnaceMaterialSplitDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<ProductionFurnaceMaterialSplit> specification = DynamicSpecifications.bySearchFilter(request, ProductionFurnaceMaterialSplit.class);
    	Page<ProductionFurnaceMaterialSplit> page = productionFurnaceMaterialSplitService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ProductionFurnaceMaterialSplitDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有投料分解记录")
	@GetMapping("/listAll")
	public Result<List<ProductionFurnaceMaterialSplitDto>> listAll(ServletRequest request) {
		Specification<ProductionFurnaceMaterialSplit> specification = DynamicSpecifications.bySearchFilter(request, ProductionFurnaceMaterialSplit.class);
		List<ProductionFurnaceMaterialSplit> productionFurnaceMaterialSplits = productionFurnaceMaterialSplitService.findByExampleSort(specification, null);
		List<ProductionFurnaceMaterialSplitDto> productionFurnaceMaterialSplitDtos = productionFurnaceMaterialSplits.stream()
					.map(productionFurnaceMaterialSplit -> toLazyDto(productionFurnaceMaterialSplit))
					.collect(Collectors.toList());
		return ResultGenerator.success(productionFurnaceMaterialSplitDtos);
	}

	@ApiOperation(value = "投料分解记录详细信息")
	@GetMapping("/{id}")
	public Result<ProductionFurnaceMaterialSplitDto> get(@PathVariable @ApiParam(value="投料分解记录id") Long id) {
		ProductionFurnaceMaterialSplit productionFurnaceMaterialSplit = productionFurnaceMaterialSplitService.get(id);
		if (null == productionFurnaceMaterialSplit)
			return ResultGenerator.fail("该投料分解记录不存在");
		return ResultGenerator.success(toEagerDto(productionFurnaceMaterialSplit));
	}

	@ApiOperation(value = "创建投料分解记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "投料分解记录对象") ProductionFurnaceMaterialSplitDto productionFurnaceMaterialSplitDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ProductionFurnaceMaterialSplit productionFurnaceMaterialSplit = fromDto(productionFurnaceMaterialSplitDto);
		productionFurnaceMaterialSplitService.saveOrUpdate(productionFurnaceMaterialSplit);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新投料分解记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "投料分解记录") ProductionFurnaceMaterialSplitDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ProductionFurnaceMaterialSplit entity = productionFurnaceMaterialSplitService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ProductionFurnaceMaterialSplit entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			productionFurnaceMaterialSplitService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除投料分解记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				productionFurnaceMaterialSplitService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ProductionFurnaceMaterialSplitDto toLazyDto(ProductionFurnaceMaterialSplit entity) {
		return modelMapper.map(entity, ProductionFurnaceMaterialSplitDto.class);
	}

	public ProductionFurnaceMaterialSplitDto toEagerDto(ProductionFurnaceMaterialSplit entity) {
		ProductionFurnaceMaterialSplitDto dto = modelMapper.map(entity, ProductionFurnaceMaterialSplitDto.class);
		return dto;
	}

	public ProductionFurnaceMaterialSplit fromDto(ProductionFurnaceMaterialSplitDto dto) {
		return modelMapper.map(dto, ProductionFurnaceMaterialSplit.class);
	}


}
