package com.sw.mes.yb.production.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.production.entity.ProductionWsmReserve;
import com.sw.mes.yb.production.dto.ProductionWsmReserveDto;
import com.sw.mes.yb.production.service.ProductionWsmReserveService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/ProductionWsmReserve")
@Slf4j
@Api(description = "圆棒WSM库存信息接口")
public class ProductionWsmReserveController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ProductionWsmReserveService productionWsmReserveService;

	@ApiOperation(value = "分页查询圆棒WSM库存信息列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ProductionWsmReserveDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<ProductionWsmReserve> specification = DynamicSpecifications.bySearchFilter(request, ProductionWsmReserve.class);
    	Page<ProductionWsmReserve> page = productionWsmReserveService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ProductionWsmReserveDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有圆棒WSM库存信息")
	@GetMapping("/listAll")
	public Result<List<ProductionWsmReserveDto>> listAll(ServletRequest request) {
		Specification<ProductionWsmReserve> specification = DynamicSpecifications.bySearchFilter(request, ProductionWsmReserve.class);
		List<ProductionWsmReserve> productionWsmReserves = productionWsmReserveService.findByExampleSort(specification, null);
		List<ProductionWsmReserveDto> productionWsmReserveDtos = productionWsmReserves.stream()
					.map(productionWsmReserve -> toLazyDto(productionWsmReserve))
					.collect(Collectors.toList());
		return ResultGenerator.success(productionWsmReserveDtos);
	}

	@ApiOperation(value = "圆棒WSM库存信息详细信息")
	@GetMapping("/{id}")
	public Result<ProductionWsmReserveDto> get(@PathVariable @ApiParam(value="圆棒WSM库存信息id") Long id) {
		ProductionWsmReserve productionWsmReserve = productionWsmReserveService.get(id);
		if (null == productionWsmReserve) {
            return ResultGenerator.fail("该圆棒WSM库存信息不存在");
        }
		return ResultGenerator.success(toEagerDto(productionWsmReserve));
	}

	@ApiOperation(value = "创建圆棒WSM库存信息")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "圆棒WSM库存信息对象") ProductionWsmReserveDto productionWsmReserveDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ProductionWsmReserve productionWsmReserve = fromDto(productionWsmReserveDto);
		productionWsmReserveService.saveOrUpdate(productionWsmReserve);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新圆棒WSM库存信息")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "圆棒WSM库存信息") ProductionWsmReserveDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if (dto.getId() == null) {
            return ResultGenerator.fail("id不能为空");
        }
            ProductionWsmReserve entity = productionWsmReserveService.get(dto.getId());
        if (entity == null) {
            return ResultGenerator.fail("原信息不存在");
        }
        try {
            ProductionWsmReserve entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			productionWsmReserveService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除圆棒WSM库存信息")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
		for (Long id : ids) {
				productionWsmReserveService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ProductionWsmReserveDto toLazyDto(ProductionWsmReserve entity) {
		return modelMapper.map(entity, ProductionWsmReserveDto.class);
	}

	public ProductionWsmReserveDto toEagerDto(ProductionWsmReserve entity) {
		ProductionWsmReserveDto dto = modelMapper.map(entity, ProductionWsmReserveDto.class);
		return dto;
	}

	public ProductionWsmReserve fromDto(ProductionWsmReserveDto dto) {
		return modelMapper.map(dto, ProductionWsmReserve.class);
	}


}
