package com.sw.mes.yb.production.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.production.entity.WeightInputDetail;
import com.sw.mes.yb.production.dto.WeightInputDetailDto;
import com.sw.mes.yb.production.service.WeightInputDetailService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/productionWeightInputDetail")
@Slf4j
@Api(description = "桶重量分配方案操作记录接口")
public class WeightInputDetailController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private WeightInputDetailService weightInputDetailService;

	@ApiOperation(value = "分页查询桶重量分配方案操作记录列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<WeightInputDetailDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<WeightInputDetail> specification = DynamicSpecifications.bySearchFilter(request, WeightInputDetail.class);
    	Page<WeightInputDetail> page = weightInputDetailService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<WeightInputDetailDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有桶重量分配方案操作记录")
	@GetMapping("/listAll")
	public Result<List<WeightInputDetailDto>> listAll(ServletRequest request) {
		Specification<WeightInputDetail> specification = DynamicSpecifications.bySearchFilter(request, WeightInputDetail.class);
		List<WeightInputDetail> weightInputDetails = weightInputDetailService.findByExampleSort(specification, null);
		List<WeightInputDetailDto> weightInputDetailDtos = weightInputDetails.stream()
					.map(weightInputDetail -> toLazyDto(weightInputDetail))
					.collect(Collectors.toList());
		return ResultGenerator.success(weightInputDetailDtos);
	}

	@ApiOperation(value = "桶重量分配方案操作记录详细信息")
	@GetMapping("/{id}")
	public Result<WeightInputDetailDto> get(@PathVariable @ApiParam(value="桶重量分配方案操作记录id") Long id) {
		WeightInputDetail weightInputDetail = weightInputDetailService.get(id);
		if (null == weightInputDetail) {
            return ResultGenerator.fail("该桶重量分配方案操作记录不存在");
        }
		return ResultGenerator.success(toEagerDto(weightInputDetail));
	}

	@ApiOperation(value = "创建桶重量分配方案操作记录")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "桶重量分配方案操作记录对象") WeightInputDetailDto weightInputDetailDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		WeightInputDetail weightInputDetail = fromDto(weightInputDetailDto);
		weightInputDetailService.saveOrUpdate(weightInputDetail);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新桶重量分配方案操作记录")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "桶重量分配方案操作记录") WeightInputDetailDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        }
        if (dto.getId() == null) {
            return ResultGenerator.fail("id不能为空");
        }
            WeightInputDetail entity = weightInputDetailService.get(dto.getId());
        if (entity == null) {
            return ResultGenerator.fail("原信息不存在");
        }
        try {
            WeightInputDetail entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			weightInputDetailService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除桶重量分配方案操作记录")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				weightInputDetailService.delete(id);
		}
		return ResultGenerator.success();
	}

	public WeightInputDetailDto toLazyDto(WeightInputDetail entity) {
		return modelMapper.map(entity, WeightInputDetailDto.class);
	}

	public WeightInputDetailDto toEagerDto(WeightInputDetail entity) {
		WeightInputDetailDto dto = modelMapper.map(entity, WeightInputDetailDto.class);
		return dto;
	}

	public WeightInputDetail fromDto(WeightInputDetailDto dto) {
		return modelMapper.map(dto, WeightInputDetail.class);
	}


}
