package com.sw.mes.yb.production.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.production.dto.ZaijingJudgmentRuleDto;
import com.sw.mes.yb.production.entity.ZaijingJudgmentRule;
import com.sw.mes.yb.production.service.ZaijingJudgmentRuleService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/zaijingJudgmentRule")
@Slf4j
@Api(description = "仔晶原料判定规则接口")
public class ZaijingJudgmentRuleController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ZaijingJudgmentRuleService zaijingJudgmentRuleService;

	@ApiOperation(value = "分页查询仔晶原料判定规则列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ZaijingJudgmentRuleDto>> listByPage(ServletRequest request, @ApiIgnore RequestPage requestPage) {
		Specification<ZaijingJudgmentRule> specification = DynamicSpecifications.bySearchFilter(request, ZaijingJudgmentRule.class);
    	Page<ZaijingJudgmentRule> page = zaijingJudgmentRuleService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ZaijingJudgmentRuleDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有仔晶原料判定规则")
	@GetMapping("/listAll")
	public Result<List<ZaijingJudgmentRuleDto>> listAll(ServletRequest request) {
		Specification<ZaijingJudgmentRule> specification = DynamicSpecifications.bySearchFilter(request, ZaijingJudgmentRule.class);
		List<ZaijingJudgmentRule> zaijingJudgmentRules = zaijingJudgmentRuleService.findByExampleSort(specification, null);
		List<ZaijingJudgmentRuleDto> zaijingJudgmentRuleDtos = zaijingJudgmentRules.stream()
					.map(zaijingJudgmentRule -> toLazyDto(zaijingJudgmentRule))
					.collect(Collectors.toList());
		return ResultGenerator.success(zaijingJudgmentRuleDtos);
	}

	@ApiOperation(value = "仔晶原料判定规则详细信息")
	@GetMapping("/{id}")
	public Result<ZaijingJudgmentRuleDto> get(@PathVariable @ApiParam(value="仔晶原料判定规则id") Integer id) {
		ZaijingJudgmentRule zaijingJudgmentRule = zaijingJudgmentRuleService.get(id);
		if (null == zaijingJudgmentRule)
			return ResultGenerator.fail("该仔晶原料判定规则不存在");
		return ResultGenerator.success(toEagerDto(zaijingJudgmentRule));
	}

	@ApiOperation(value = "创建仔晶原料判定规则")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "仔晶原料判定规则对象") ZaijingJudgmentRuleDto zaijingJudgmentRuleDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ZaijingJudgmentRule zaijingJudgmentRule = fromDto(zaijingJudgmentRuleDto);
		zaijingJudgmentRule.setGmtCreate(new Date());
		zaijingJudgmentRule.setUserCreate(BaseContextHandler.getUsername());
		zaijingJudgmentRuleService.saveOrUpdate(zaijingJudgmentRule);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新仔晶原料判定规则")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "仔晶原料判定规则") ZaijingJudgmentRuleDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ZaijingJudgmentRule entity = zaijingJudgmentRuleService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ZaijingJudgmentRule entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			entity.setGmtUpdate(new Date());
			entity.setUserUpdate(BaseContextHandler.getUsername());
			zaijingJudgmentRuleService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除仔晶原料判定规则")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				zaijingJudgmentRuleService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ZaijingJudgmentRuleDto toLazyDto(ZaijingJudgmentRule entity) {
		return modelMapper.map(entity, ZaijingJudgmentRuleDto.class);
	}

	public ZaijingJudgmentRuleDto toEagerDto(ZaijingJudgmentRule entity) {
		ZaijingJudgmentRuleDto dto = modelMapper.map(entity, ZaijingJudgmentRuleDto.class);
		return dto;
	}

	public ZaijingJudgmentRule fromDto(ZaijingJudgmentRuleDto dto) {
		return modelMapper.map(dto, ZaijingJudgmentRule.class);
	}


}
