/**
 * There are <a href="https://github.com/ketayao/keta-custom">keta-custom</a> code generation
 */
package com.sw.mes.yb.production.entity;

import lombok.Data;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.math.BigDecimal;
import java.util.Date;

/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@Data
@Entity
@Table(name="production_loading_task_log")
@DynamicUpdate
@DynamicInsert
public class LoadingTaskLog{
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

	//任务ID
	@Column
	private Long loadingTaskId;

	/**
	 * 设备号
	 */
    @Column(length=100)
    private String equipmentCode;

	/**
	 * 炉次号
	 */
    @Column(length=100)
    private String lotNo;

	/**
	 * 所属系列
	 */
    @Column(length=50)
    private String series;

	/**
	 * 所属模块
	 */
    @Column(length=50)
    private String area;

	/**
	 * 叫料类型【0-初始装料;】
	 */
    @Column(length=3)
    private Integer rawType;

	/**
	 * 创建时间
	 */
    @Column(length=19)
	@Temporal(TemporalType.TIMESTAMP)
    private Date gmtCreate;

	/**
	 * 预计装料时间
	 */
    @Column(length=19)
	@Temporal(TemporalType.TIMESTAMP)
    private Date etTime;

	/**
	 * 延时任务开始时间
	 */
    @Column(length=19)
	@Temporal(TemporalType.TIMESTAMP)
    private Date delayedStartTime;

	/**
	 * 状态【0-新建;1-生产;2-已取消;3-自动完工;4-人工完工;】
	 */
    @Column(length=3)
    private Integer rawStatus;

	/**
	 * 炉台类型【0-量产;1-在建;2-研发;】
	 */
    @Column(length=3)
    private Integer furnaceType;

	/**
	 * SAP订单号
	 */
    @Column(length=100)
    private String sapNo;

	/**
	 * 料号
	 */
    @Column(length=100)
    private String productCode;

	/**
	 * 产品等级
	 */
    @Column(length=50)
    private String productGrade;

	/**
	 * 规格
	 */
    @Column(length=50)
    private String specs;

	/**
	 * 掺杂剂
	 */
    @Column(length=10)
    private String dopant;

	/**
	 * 产品类别
	 */
    @Column(length=50)
    private String productType;

	/**
	 * 配方方案ID(production_circle_recipe.id)
	 */
    @Column(length=19)
    private Long recipeId;

	/**
	 * 重量方案ID(production_circle_weigth.id)
	 */
    @Column(length=19)
    private Long weigthId;

	/**
	 * 更换配方人员
	 */
    @Column(length=50)
    private String changeRecipeUserName;

	/**
	 * 更换配方时间
	 */
    @Column(length=19)
	@Temporal(TemporalType.TIMESTAMP)
    private Date changeRecipeTime;

	/**
	 * 操作人
	 */
    @Column(length=60)
    private String userName;

	/**
	 * 操作时间
	 */
    @Column(length=19)
	@Temporal(TemporalType.TIMESTAMP)
    private Date operationTime;

	/**
	 * 操作原因
	 */
    @Column(length=255)
    private String operationCause;

	/**
	 * 配料人
	 */
    @Column(length=60)
    private String recipeUserName;

	/**
	 * 配料时间
	 */
    @Column(length=19)
	@Temporal(TemporalType.TIMESTAMP)
    private Date recipeTime;

	/**
	 * 配料总重量
	 */
    @Column(length=19)
    private BigDecimal weight;


	/**
	 * 坩埚高度
	 */
	@Column(length=10)
	private String crucibleHeight;

	/**
	 * 配料实际重量
	 */
	@Column(length=19)
	private BigDecimal realWeight;

	/**
	 * 排程ID
	 */
	@Column(length=11)
	private Long scheduleId;

	/**
	 * 产品简称
	 */
	@Column(length=50)
	private String simpleName;

	/**
	 * 载具编号
	 */
	@Column(length=50)
	private String carrierNo;

	/**
	 * 确认投料(0:未确认, 1:已确认)
	 */
	@Column(length=4)
	private Integer confirm = 0;

	/**
	 * 确认投料时间
	 */
	@Column(nullable=false, length=19)
	@Temporal(TemporalType.TIMESTAMP)
	private Date confirmTime;


	public LoadingTaskLog() {
	}

	public LoadingTaskLog(Long id) {
		this.id = id;
	}
}
