package com.sw.mes.yb.report.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.report.entity.ExpansionBase;
import com.sw.mes.yb.report.dto.ExpansionBaseDto;
import com.sw.mes.yb.report.service.ExpansionBaseService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/expansionBase")
@Slf4j
@Api(description = "ExpansionBase接口")
public class ExpansionBaseController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ExpansionBaseService expansionBaseService;

	@ApiOperation(value = "分页查询ExpansionBase列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ExpansionBaseDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<ExpansionBase> specification = DynamicSpecifications.bySearchFilter(request, ExpansionBase.class);
    	Page<ExpansionBase> page = expansionBaseService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ExpansionBaseDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有ExpansionBase")
	@GetMapping("/listAll")
	public Result<List<ExpansionBaseDto>> listAll(ServletRequest request) {
		Specification<ExpansionBase> specification = DynamicSpecifications.bySearchFilter(request, ExpansionBase.class);
		List<ExpansionBase> expansionBases = expansionBaseService.findByExampleSort(specification, null);
		List<ExpansionBaseDto> expansionBaseDtos = expansionBases.stream()
					.map(expansionBase -> toLazyDto(expansionBase))
					.collect(Collectors.toList());
		return ResultGenerator.success(expansionBaseDtos);
	}

	@ApiOperation(value = "ExpansionBase详细信息")
	@GetMapping("/{id}")
	public Result<ExpansionBaseDto> get(@PathVariable @ApiParam(value="ExpansionBaseid") Long id) {
		ExpansionBase expansionBase = expansionBaseService.get(id);
		if (null == expansionBase)
			return ResultGenerator.fail("该ExpansionBase不存在");
		return ResultGenerator.success(toEagerDto(expansionBase));
	}

	@ApiOperation(value = "创建ExpansionBase")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "ExpansionBase对象") ExpansionBaseDto expansionBaseDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ExpansionBase expansionBase = fromDto(expansionBaseDto);
		expansionBaseService.saveOrUpdate(expansionBase);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新ExpansionBase")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "ExpansionBase") ExpansionBaseDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ExpansionBase entity = expansionBaseService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ExpansionBase entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			expansionBaseService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除ExpansionBase")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				expansionBaseService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ExpansionBaseDto toLazyDto(ExpansionBase entity) {
		return modelMapper.map(entity, ExpansionBaseDto.class);
	}

	public ExpansionBaseDto toEagerDto(ExpansionBase entity) {
		ExpansionBaseDto dto = modelMapper.map(entity, ExpansionBaseDto.class);
		return dto;
	}

	public ExpansionBase fromDto(ExpansionBaseDto dto) {
		return modelMapper.map(dto, ExpansionBase.class);
	}


}
