package com.sw.mes.yb.report.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.report.entity.ShoulderSeedingRate;
import com.sw.mes.yb.report.dto.ShoulderSeedingRateDto;
import com.sw.mes.yb.report.service.ShoulderSeedingRateService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/shoulderSeedingRate")
@Slf4j
@Api(description = "ShoulderSeedingRate接口")
public class ShoulderSeedingRateController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private ShoulderSeedingRateService shoulderSeedingRateService;

	@ApiOperation(value = "分页查询ShoulderSeedingRate列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<ShoulderSeedingRateDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<ShoulderSeedingRate> specification = DynamicSpecifications.bySearchFilter(request, ShoulderSeedingRate.class);
    	Page<ShoulderSeedingRate> page = shoulderSeedingRateService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<ShoulderSeedingRateDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有ShoulderSeedingRate")
	@GetMapping("/listAll")
	public Result<List<ShoulderSeedingRateDto>> listAll(ServletRequest request) {
		Specification<ShoulderSeedingRate> specification = DynamicSpecifications.bySearchFilter(request, ShoulderSeedingRate.class);
		List<ShoulderSeedingRate> shoulderSeedingRates = shoulderSeedingRateService.findByExampleSort(specification, null);
		List<ShoulderSeedingRateDto> shoulderSeedingRateDtos = shoulderSeedingRates.stream()
					.map(shoulderSeedingRate -> toLazyDto(shoulderSeedingRate))
					.collect(Collectors.toList());
		return ResultGenerator.success(shoulderSeedingRateDtos);
	}

	@ApiOperation(value = "ShoulderSeedingRate详细信息")
	@GetMapping("/{id}")
	public Result<ShoulderSeedingRateDto> get(@PathVariable @ApiParam(value="ShoulderSeedingRateid") Long id) {
		ShoulderSeedingRate shoulderSeedingRate = shoulderSeedingRateService.get(id);
		if (null == shoulderSeedingRate)
			return ResultGenerator.fail("该ShoulderSeedingRate不存在");
		return ResultGenerator.success(toEagerDto(shoulderSeedingRate));
	}

	@ApiOperation(value = "创建ShoulderSeedingRate")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "ShoulderSeedingRate对象") ShoulderSeedingRateDto shoulderSeedingRateDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		ShoulderSeedingRate shoulderSeedingRate = fromDto(shoulderSeedingRateDto);
		shoulderSeedingRateService.saveOrUpdate(shoulderSeedingRate);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新ShoulderSeedingRate")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "ShoulderSeedingRate") ShoulderSeedingRateDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            ShoulderSeedingRate entity = shoulderSeedingRateService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            ShoulderSeedingRate entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			shoulderSeedingRateService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除ShoulderSeedingRate")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				shoulderSeedingRateService.delete(id);
		}
		return ResultGenerator.success();
	}

	public ShoulderSeedingRateDto toLazyDto(ShoulderSeedingRate entity) {
		return modelMapper.map(entity, ShoulderSeedingRateDto.class);
	}

	public ShoulderSeedingRateDto toEagerDto(ShoulderSeedingRate entity) {
		ShoulderSeedingRateDto dto = modelMapper.map(entity, ShoulderSeedingRateDto.class);
		return dto;
	}

	public ShoulderSeedingRate fromDto(ShoulderSeedingRateDto dto) {
		return modelMapper.map(dto, ShoulderSeedingRate.class);
	}


}
