package com.sw.mes.yb.report.controller;

import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.yb.report.dto.SuccessCrystalBaseDto;
import com.sw.mes.yb.report.entity.SuccessCrystalBase;
import com.sw.mes.yb.report.service.SuccessCrystalBaseService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/successCrystalBase")
@Slf4j
@Api(description = "successCrystalBase接口")
public class SuccessCrystalBaseController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private SuccessCrystalBaseService successCrystalBaseService;

	@ApiOperation(value = "分页查询SuccessCrystalBase列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<SuccessCrystalBaseDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<SuccessCrystalBase> specification = DynamicSpecifications.bySearchFilter(request, SuccessCrystalBase.class);
    	Page<SuccessCrystalBase> page = successCrystalBaseService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<SuccessCrystalBaseDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有SuccessCrystalBase")
	@GetMapping("/listAll")
	public Result<List<SuccessCrystalBaseDto>> listAll(ServletRequest request) {
		Specification<SuccessCrystalBase> specification = DynamicSpecifications.bySearchFilter(request, SuccessCrystalBase.class);
		List<SuccessCrystalBase> successCrystalBases = successCrystalBaseService.findByExampleSort(specification, null);
		List<SuccessCrystalBaseDto> successCrystalBaseDtos = successCrystalBases.stream()
					.map(successCrystalBase -> toLazyDto(successCrystalBase))
					.collect(Collectors.toList());
		return ResultGenerator.success(successCrystalBaseDtos);
	}

	@ApiOperation(value = "SuccessCrystalBase详细信息")
	@GetMapping("/{id}")
	public Result<SuccessCrystalBaseDto> get(@PathVariable @ApiParam(value="SuccessCrystalBaseid") Long id) {
		SuccessCrystalBase successCrystalBase = successCrystalBaseService.get(id);
		if (null == successCrystalBase)
			return ResultGenerator.fail("该SuccessCrystalBase不存在");
		return ResultGenerator.success(toEagerDto(successCrystalBase));
	}

	@ApiOperation(value = "创建SuccessCrystalBase")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "SuccessCrystalBase对象") SuccessCrystalBaseDto successCrystalBaseDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		SuccessCrystalBase successCrystalBase = fromDto(successCrystalBaseDto);
		successCrystalBaseService.saveOrUpdate(successCrystalBase);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新SuccessCrystalBase")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "SuccessCrystalBase") SuccessCrystalBaseDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            SuccessCrystalBase entity = successCrystalBaseService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            SuccessCrystalBase entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			successCrystalBaseService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除SuccessCrystalBase")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
			successCrystalBaseService.delete(id);
		}
		return ResultGenerator.success();
	}

	public SuccessCrystalBaseDto toLazyDto(SuccessCrystalBase entity) {
		return modelMapper.map(entity, SuccessCrystalBaseDto.class);
	}

	public SuccessCrystalBaseDto toEagerDto(SuccessCrystalBase entity) {
		SuccessCrystalBaseDto dto = modelMapper.map(entity, SuccessCrystalBaseDto.class);
		return dto;
	}

	public SuccessCrystalBase fromDto(SuccessCrystalBaseDto dto) {
		return modelMapper.map(dto, SuccessCrystalBase.class);
	}


}
