package com.sw.mes.yb.turnstore.controller;

import com.sw.mes.core.context.BaseContextHandler;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import com.sw.mes.yb.turnstore.entity.MaterialTrayBinding;
import com.sw.mes.yb.turnstore.dto.MaterialTrayBindingDto;
import com.sw.mes.yb.turnstore.service.MaterialTrayBindingService;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/materialTrayBinding")
@Slf4j
@Api(description = "原料料盒托盘绑定接口")
public class MaterialTrayBindingController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private MaterialTrayBindingService materialTrayBindingService;

	@ApiOperation(value = "分页查询原料料盒托盘绑定列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<MaterialTrayBindingDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<MaterialTrayBinding> specification = DynamicSpecifications.bySearchFilter(request, MaterialTrayBinding.class);
    	Page<MaterialTrayBinding> page = materialTrayBindingService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<MaterialTrayBindingDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有原料料盒托盘绑定")
	@GetMapping("/listAll")
	public Result<List<MaterialTrayBindingDto>> listAll(ServletRequest request) {
		Specification<MaterialTrayBinding> specification = DynamicSpecifications.bySearchFilter(request, MaterialTrayBinding.class);
		List<MaterialTrayBinding> materialTrayBindings = materialTrayBindingService.findByExampleSort(specification, null);
		List<MaterialTrayBindingDto> materialTrayBindingDtos = materialTrayBindings.stream()
					.map(materialTrayBinding -> toLazyDto(materialTrayBinding))
					.collect(Collectors.toList());
		return ResultGenerator.success(materialTrayBindingDtos);
	}

	@ApiOperation(value = "原料料盒托盘绑定详细信息")
	@GetMapping("/{id}")
	public Result<MaterialTrayBindingDto> get(@PathVariable @ApiParam(value="原料料盒托盘绑定id") Long id) {
		MaterialTrayBinding materialTrayBinding = materialTrayBindingService.get(id);
		if (null == materialTrayBinding)
			return ResultGenerator.fail("该原料料盒托盘绑定不存在");
		return ResultGenerator.success(toEagerDto(materialTrayBinding));
	}

	@ApiOperation(value = "创建原料料盒托盘绑定")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "原料料盒托盘绑定对象") MaterialTrayBindingDto materialTrayBindingDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		MaterialTrayBinding materialTrayBinding = fromDto(materialTrayBindingDto);
		materialTrayBindingService.saveOrUpdate(materialTrayBinding);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新原料料盒托盘绑定")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "原料料盒托盘绑定") MaterialTrayBindingDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            MaterialTrayBinding entity = materialTrayBindingService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            MaterialTrayBinding entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			materialTrayBindingService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除原料料盒托盘绑定")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Long[] ids) {
		for (Long id : ids) {
			materialTrayBindingService.delete(id);
		}
		return ResultGenerator.success();
	}

	public MaterialTrayBindingDto toLazyDto(MaterialTrayBinding entity) {
		return modelMapper.map(entity, MaterialTrayBindingDto.class);
	}

	public MaterialTrayBindingDto toEagerDto(MaterialTrayBinding entity) {
		MaterialTrayBindingDto dto = modelMapper.map(entity, MaterialTrayBindingDto.class);
		return dto;
	}

	public MaterialTrayBinding fromDto(MaterialTrayBindingDto dto) {
		return modelMapper.map(dto, MaterialTrayBinding.class);
	}


}
