package com.sw.mes.yb.wms.controller;

import com.sw.mes.base.system.entity.InventoryManageLog;
import com.sw.mes.core.response.ResponsePage;
import com.sw.mes.core.response.Result;
import com.sw.mes.core.response.ResultGenerator;
import com.sw.mes.core.utils.BeanUtil;
import com.sw.mes.core.utils.DynamicSpecifications;
import com.sw.mes.core.utils.PageUtil;
import com.sw.mes.core.utils.RequestPage;
import com.sw.mes.core.dto.mes.pc.yb.InventoryManageLogDto;
import com.sw.mes.base.system.service.InventoryManageLogService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.modelmapper.ModelMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.ServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;


/**
* @Author
* @Description：
* @Date: Create in
* @Modified By:
*/
@RestController
@RequestMapping("/InventoryManageLog")
@Slf4j
@Api(description = "立库库存管理记录接口")
public class InventoryManageLogController {
	@Autowired
	ModelMapper modelMapper;

	@Autowired
	private InventoryManageLogService inventoryManageLogService;

	@ApiOperation(value = "分页查询InventoryManageLog列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "rows", value = "行数", dataType = "integer", paramType = "query"),
		@ApiImplicitParam(name = "page", value = "页码", dataType = "integer", paramType = "query"),
	})
	@GetMapping("/listByPage")
	public Result<ResponsePage<InventoryManageLogDto>> listByPage(ServletRequest request,@ApiIgnore RequestPage requestPage) {
		Specification<InventoryManageLog> specification = DynamicSpecifications.bySearchFilter(request, InventoryManageLog.class);
    	Page<InventoryManageLog> page = inventoryManageLogService.findByExample(specification, PageUtil.createPageable(requestPage));
		List<InventoryManageLogDto> dtos = page.getContent().stream()
                .map(entity -> toLazyDto(entity))
                .collect(Collectors.toList());
		ResponsePage responsePage = new ResponsePage(dtos, page.getTotalElements());
        return ResultGenerator.success(responsePage);
	}

	@ApiOperation(value = "获取所有InventoryManageLog")
	@GetMapping("/listAll")
	public Result<List<InventoryManageLogDto>> listAll(ServletRequest request) {
		Specification<InventoryManageLog> specification = DynamicSpecifications.bySearchFilter(request, InventoryManageLog.class);
		List<InventoryManageLog> inventoryManageLogs = inventoryManageLogService.findByExampleSort(specification, null);
		List<InventoryManageLogDto> inventoryManageLogDtos = inventoryManageLogs.stream()
					.map(inventoryManageLog -> toLazyDto(inventoryManageLog))
					.collect(Collectors.toList());
		return ResultGenerator.success(inventoryManageLogDtos);
	}

	@ApiOperation(value = "InventoryManageLog详细信息")
	@GetMapping("/{id}")
	public Result<InventoryManageLogDto> get(@PathVariable @ApiParam(value="InventoryManageLogid") Long id) {
		InventoryManageLog inventoryManageLog = inventoryManageLogService.get(id);
		if (null == inventoryManageLog)
			return ResultGenerator.fail("该InventoryManageLog不存在");
		return ResultGenerator.success(toEagerDto(inventoryManageLog));
	}

	@ApiOperation(value = "创建InventoryManageLog")
	@PostMapping("/create")
	public Result create(@RequestBody @Valid @ApiParam(value = "InventoryManageLog对象") InventoryManageLogDto inventoryManageLogDto, BindingResult bindingResult) {
		if (bindingResult.hasErrors()) {
			return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
		}
		InventoryManageLog inventoryManageLog = fromDto(inventoryManageLogDto);
		inventoryManageLogService.saveOrUpdate(inventoryManageLog);
		return ResultGenerator.success();
	}

    @ApiOperation(value = "更新InventoryManageLog")
    @PostMapping("/update")
    public Result update(@RequestBody @Valid @ApiParam(value = "InventoryManageLog") InventoryManageLogDto dto, BindingResult bindingResult) {
        if (bindingResult.hasErrors())
            return ResultGenerator.fail(bindingResult.getFieldError().getDefaultMessage());
        if (dto.getId() == null)
            return ResultGenerator.fail("id不能为空");
            InventoryManageLog entity = inventoryManageLogService.get(dto.getId());
        if (entity == null)
            return ResultGenerator.fail("原信息不存在");
        try {
            InventoryManageLog entityfromFto = fromDto(dto);
			BeanUtils.copyProperties(entityfromFto, entity, BeanUtil.getIgnoreProperties(entityfromFto, null, null));
			inventoryManageLogService.saveOrUpdate(entity);
            return ResultGenerator.success();
        } catch (Exception e) {
            return ResultGenerator.fail(e.getMessage());
        }
    }

	@ApiOperation(value = "删除InventoryManageLog")
	@PostMapping("/delete")
	public Result remove(@ApiParam(value = "id集合,逗号分隔,如2,4", type = "query") @RequestParam Integer[] ids) {
		for (Integer id : ids) {
				inventoryManageLogService.delete(id);
		}
		return ResultGenerator.success();
	}

	public InventoryManageLogDto toLazyDto(InventoryManageLog entity) {
		return modelMapper.map(entity, InventoryManageLogDto.class);
	}

	public InventoryManageLogDto toEagerDto(InventoryManageLog entity) {
		InventoryManageLogDto dto = modelMapper.map(entity, InventoryManageLogDto.class);
		return dto;
	}

	public InventoryManageLog fromDto(InventoryManageLogDto dto) {
		return modelMapper.map(dto, InventoryManageLog.class);
	}


}
